#!/usr/bin/env python3
###
### Script to check water conservation in the IPSL coupled model
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
# SVN Information
SVN = {
    'Author'  : "$Author$",
    'Date'    : "$Date$",
    'Revision': "$Revision$",
    'Id'      : "$Id: ATM_waterbudget.py 6508 2023-06-13 10:58:38Z omamce $",
    'HeadURL' : "$HeadUrl: svn+ssh://omamce@forge.ipsl.jussieu.fr/ipsl/forge/projets/igcmg/svn/TOOLS/WATER_BUDGET/ATM_waterbudget.py $"
    }

###
## Import system modules
import sys
import os
import configparser

## Import needed scientific modules
import numpy as np
import xarray as xr

## Import local modules
import WaterUtils as wu
import lmdz

## Read command line arguments
## ---------------------------
print ( "Name of Python script:", sys.argv[0] )
IniFile = sys.argv[1]

# Test existence of IniFile
if not os.path.exists (IniFile ) :
    raise FileExistsError ( f"File not found : {IniFile = }" )

if 'full' in IniFile or 'ATM' in IniFile :
    FullIniFile = IniFile
else :
    FullIniFile = 'ATM_' + IniFile

print ("Output file : ", FullIniFile )

## Experiment parameters
## --------------------
dpar = wu.ReadConfig ( IniFile )

## Configure all needed parameter from existant parameters
## -------------------------------------------------------
dpar = wu.SetDatesAndFiles ( dpar )

## Output file with water budget diagnostics
## -----------------------------------------
f_out = dpar['Files']['f_out']

## Put dpar values in local namespace
## ----------------------------------
for Section in dpar.keys () : 
    print ( f'\nReading [{Section}]' )
    for VarName in dpar[Section].keys() :
        globals()[VarName] = dpar[Section][VarName]
        print ( f'    {VarName:21} set to : {globals()[VarName]}' )

## Debuging and timer
Timer = wu.functools.partial (wu.Timer, debug=Debug, timer=Timing)

## Useful functions
## ----------------
if repr(readPrec) == "<class 'numpy.float64'>" or readPrec == float :
    def rprec (ptab) :
        '''This version does nothing

        rprec may be used to reduce floating precision when reading history files
        '''
        return ptab
else                 :
    def rprec (ptab) :
        '''Returns float with a different precision'''
        return ptab.astype(readPrec).astype(float)

def kg2Sv    (pval, rho=ATM_RHO) :
    '''From kg to Sverdrup'''
    return pval/dtime_sec*1.0e-6/rho

def kg2myear (pval, rho=ATM_RHO) :
    '''From kg to m/year'''
    return pval/ATM_aire_sea_tot/rho/NbYear

def var2prt (pvar, small=False, rho=ATM_RHO) :
    '''Formats value for printing'''
    if small :  return pvar, kg2Sv (pvar, rho=rho)*1000., kg2myear (pvar, rho=rho)*1000
    else     :  return pvar, kg2Sv (pvar, rho=rho)      , kg2myear (pvar, rho=rho)

def prtFlux (Desc, pvar, Form='F', small=False, rho=ATM_RHO, width=15) :
    '''Pretty print of formatted value'''
    if small :
        if Form in ['f', 'F'] : ff=" {:14.6e} kg | {:12.4f} mSv | {:12.4f} mm/year "
        if Form in ['e', 'E'] : ff=" {:14.6e} kg | {:12.4e} mSv | {:12.4e} mm/year "
    else :
        if Form in ['f', 'F'] : ff=" {:14.6e} kg | {:12.4f} Sv  | {:12.4f} m/year  "
        if Form in ['e', 'E'] : ff=" {:14.6e} kg | {:12.4e} Sv  | {:12.4e} m/year  "
    echo ( (' {:>{width}} = ' +ff).format (Desc, *var2prt (pvar, small=small, rho=rho), width=width ) )

def echo (string, end='\n') :
    '''Function to print to stdout *and* output file'''
    print ( str(string), end=end  )
    sys.stdout.flush ()
    f_out.write ( str(string) + end )
    f_out.flush ()

## Open history files
## ------------------
d_ATM_his = xr.open_dataset ( file_ATM_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
if SRF :
    d_SRF_his = xr.open_dataset ( file_SRF_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
    if Routing == 'SECHIBA' : d_RUN_his = d_SRF_his
    if Routing == 'SIMPLE'  : d_RUN_his = xr.open_dataset ( file_RUN_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()

echo ( f'{file_ATM_his = }' )
if SRF :
    echo ( f'{file_SRF_his = }' )
    
## Compute run length
## ------------------
dtime = ( d_ATM_his.time_counter_bounds.max() - d_ATM_his.time_counter_bounds.min() )
echo ( f'\nRun length : {(dtime/np.timedelta64(1, "D")).values:8.2f} days' )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

##-- Compute length of each period
dtime_per = (d_ATM_his.time_counter_bounds[:,-1] -  d_ATM_his.time_counter_bounds[:,0] )
echo ( '\nPeriods lengths (days) : {:} days'.format ( (dtime_per/np.timedelta64(1, "D")).values ) )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_ATM_his.time_counter,] )
dtime_per_sec.attrs['unit'] = 's'

##-- Number of years (approximative)
NbYear = dtime_sec / YEAR_LENGTH

## Define restart periods and file names
## -------------------------------------
dpar = wu.SetRestartNames ( dpar, f_out) 

## Put dpar values in local namespace
## ----------------------------------
for Section in dpar.keys () : 
    print ( f'\nReading [{Section}]' )
    for VarName in dpar[Section].keys() :
        locals()[VarName] = dpar[Section][VarName]
        print ( f'    {VarName:21} set to : {locals()[VarName]}' )
        
## Extract restart files from tar
## ----------------------------------

liste_beg = [file_ATM_beg, ]
liste_end = [file_ATM_end, ]
if file_DYN_beg : liste_beg.append ( file_DYN_beg )
if file_DYN_end : liste_end.append ( file_DYN_end )
if file_SRF_beg : liste_beg.append ( file_SRF_beg )
if file_SRF_end : liste_end.append ( file_SRF_end )

if ICO :
    if not file_DYN_aire :
        file_DYN_aire = os.path.join ( R_IN, 'ATM', 'GRID', ResolAtm+'_grid.nc' )
    dpar['Files']['file_DYN_aire'] = file_DYN_aire

if SRF and Routing == 'SIMPLE' :
    liste_beg.append ( file_RUN_beg )
    liste_end.append ( file_RUN_end )

echo ( '\nExtract restart files from tar : ATM, ICO', end='')
if SRF : echo ( ' and SRF')
else   : echo (' ')

## Write the full configuration
## ----------------------------
params_out = open (FullIniFile, 'w', encoding = 'utf-8')
params = wu.dict2config ( dpar )
params.write ( params_out )
params_out.close ()
    
@Timer
def extract ( file_name=file_ATM_beg, tar_restart=tar_restart_end, file_dir_comp=FileDir ) :
    '''
    Extract restart files from a tar
    '''
    echo ( f'----------')
    error_count = 0
    echo ( f'file to extract : {file_name = }' )
    if os.path.exists ( os.path.join (FileDir, file_name) ) :
        echo ( f'file found      : {file_name = }' )
    else :
        echo ( f'file not found  : {file_name = }' )
        base_resFile = os.path.basename (file_name)
        if os.path.exists ( tar_restart ) :
            command =  f'cd {file_dir_comp} ; tar xf {tar_restart} {base_resFile}'
            echo ( f'{command = }' )
            err = os.system ( command )
            if err != 0 :
                if ContinueOnError :
                    error_count += 1
                    echo ( f'****** Command failed : {command}' )
                    echo ( '****** Trying to continue' )
                    echo ( ' ')
                else :
                    raise OSError ( f'**** command failed : {command} - Stopping' )
            else :
                echo ( f'tar done : {base_resFile}' )
        else :
            echo ( f'****** Tar restart file {tar_restart = } not found ' )
            if ContinueOnError :
                error_count += 1
                echo ( f'****** Command failed : {command}' )
                echo (  '****** Trying to continue' )
                echo (  ' ')
            else :
                raise OSError ( f'****** tar file not found {tar_restart = } - Stopping' )
    return error_count

ErrorCount = 0

ErrorCount += extract ( file_name=file_ATM_beg, tar_restart=tar_restart_beg_ATM, file_dir_comp=FileDir )
ErrorCount += extract ( file_name=file_DYN_beg, tar_restart=tar_restart_beg_DYN, file_dir_comp=FileDir )

ErrorCount += extract ( file_name=file_ATM_end, tar_restart=tar_restart_end_ATM, file_dir_comp=FileDir )
ErrorCount += extract ( file_name=file_DYN_end, tar_restart=tar_restart_end_DYN, file_dir_comp=FileDir )

if SRF :
    ErrorCount += extract ( file_name=file_SRF_beg, tar_restart=tar_restart_beg_SRF, file_dir_comp=FileDir )
    ErrorCount += extract ( file_name=file_SRF_end, tar_restart=tar_restart_end_SRF, file_dir_comp=FileDir )

    if Routing == 'SIMPLE' :
        ErrorCount += extract ( file_name=file_RUN_beg, tar_restart=tar_restart_beg_RUN, file_dir_comp=FileDir )
        ErrorCount += extract ( file_name=file_RUN_end, tar_restart=tar_restart_end_RUN, file_dir_comp=FileDir )

##-- Exit in case of error in the opening file phase
if ErrorCount > 0 :
    echo ( '  ' )
    raise RuntimeError ( f'**** Some files missing - Stopping - {ErrorCount = }' )

##
echo ('\nOpening ATM SRF and ICO restart files')
d_ATM_beg = xr.open_dataset ( os.path.join (FileDir, file_ATM_beg), decode_times=False, decode_cf=True ).squeeze()
d_ATM_end = xr.open_dataset ( os.path.join (FileDir, file_ATM_end), decode_times=False, decode_cf=True ).squeeze()
if SRF :
    d_SRF_beg = xr.open_dataset ( os.path.join (FileDir, file_SRF_beg), decode_times=False, decode_cf=True ).squeeze()
    d_SRF_end = xr.open_dataset ( os.path.join (FileDir, file_SRF_end), decode_times=False, decode_cf=True ).squeeze()
d_DYN_beg = xr.open_dataset ( os.path.join (FileDir, file_DYN_beg), decode_times=False, decode_cf=True ).squeeze()
d_DYN_end = xr.open_dataset ( os.path.join (FileDir, file_DYN_end), decode_times=False, decode_cf=True ).squeeze()

if SRF :
    for var in d_SRF_beg.variables :
        d_SRF_beg[var] = d_SRF_beg[var].where ( d_SRF_beg[var]<1.e20, 0.)
        d_SRF_end[var] = d_SRF_end[var].where ( d_SRF_end[var]<1.e20, 0.)

    if Routing == 'SIMPLE' :
        d_RUN_beg = xr.open_dataset ( os.path.join (FileDir, file_RUN_beg), decode_times=False, decode_cf=True ).squeeze()
        d_RUN_end = xr.open_dataset ( os.path.join (FileDir, file_RUN_end), decode_times=False, decode_cf=True ).squeeze()

@Timer
def to_cell ( dd, newname='cell' ) :
    '''Set space dimension to newname
    '''
    for oldname in [ 'cell_mesh', 'y', 'points_physiques' ] :
        if oldname in dd.dims and oldname != newname :
            dd = dd.rename ( {oldname : newname} )
    return dd

d_ATM_beg = to_cell ( d_ATM_beg )
d_ATM_end = to_cell ( d_ATM_end )
if SRF :
    d_SRF_beg = to_cell ( d_SRF_beg )
    d_SRF_end = to_cell ( d_SRF_end )
d_DYN_beg = to_cell ( d_DYN_beg )
d_DYN_end = to_cell ( d_DYN_end )

if SRF and Routing == 'SIMPLE' :
    d_RUN_beg = to_cell ( d_RUN_beg )
    d_RUN_end = to_cell ( d_RUN_end )

d_ATM_his = to_cell ( d_ATM_his )
if SRF : d_SRF_his = to_cell ( d_SRF_his )

echo ( f'{file_ATM_beg = }' )
echo ( f'{file_ATM_end = }' )
echo ( f'{file_DYN_beg = }' )
echo ( f'{file_DYN_end = }' )
if SRF :
    echo ( f'{file_SRF_beg = }' )
    echo ( f'{file_SRF_end = }' )
    if Routing == 'SIMPLE' :
        echo ( f'{file_RUN_beg = }' )
        echo ( f'{file_RUN_end = }' )

# ATM grid with cell surfaces
if LMDZ :
    echo ('ATM grid with cell surfaces : LMDZ')
    ATM_lat       = lmdz.geo2point (   rprec (d_ATM_his ['lat'])+0*rprec (d_ATM_his ['lon']), dim1d='cell' )
    ATM_lon       = lmdz.geo2point ( 0*rprec (d_ATM_his ['lat'])+  rprec (d_ATM_his ['lon']), dim1d='cell' )
    ATM_aire      = lmdz.geo2point ( rprec (d_ATM_his ['aire']     [0]), cumul_poles=True, dim1d='cell' )
    ATM_fter      = lmdz.geo2point ( rprec (d_ATM_his ['fract_ter'][0]), dim1d='cell' )
    ATM_foce      = lmdz.geo2point ( rprec (d_ATM_his ['fract_oce'][0]), dim1d='cell' )
    ATM_fsic      = lmdz.geo2point ( rprec (d_ATM_his ['fract_sic'][0]), dim1d='cell' )
    ATM_flic      = lmdz.geo2point ( rprec (d_ATM_his ['fract_lic'][0]), dim1d='cell' )
    if SRF :
        SRF_lat       = lmdz.geo2point (   rprec (d_SRF_his ['lat'])+0*rprec (d_SRF_his ['lon']), dim1d='cell' )
        SRF_lon       = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat'])+  rprec (d_SRF_his ['lon']), dim1d='cell' )
        SRF_aire      = lmdz.geo2point ( rprec (d_SRF_his ['Areas']) * rprec (d_SRF_his ['Contfrac']), dim1d='cell', cumul_poles=True )
        SRF_areas     = lmdz.geo2point ( rprec (d_SRF_his ['Areas'])  ,  dim1d='cell', cumul_poles=True )
        SRF_contfrac  = lmdz.geo2point ( rprec (d_SRF_his ['Contfrac']), dim1d='cell' )

if ICO :
    if ATM_HIS == 'latlon' :
        echo ( 'ATM areas and fractions on LATLON grid' )
        if 'lat_dom_out' in d_ATM_his.variables :
            ATM_lat  = lmdz.geo2point (   rprec (d_ATM_his ['lat_dom_out'])+0*rprec (d_ATM_his ['lon_dom_out']), dim1d='cell' )
            ATM_lon  = lmdz.geo2point ( 0*rprec (d_ATM_his ['lat_dom_out'])+  rprec (d_ATM_his ['lon_dom_out']), dim1d='cell' )
        else :
            ATM_lat  = lmdz.geo2point (   rprec (d_ATM_his ['lat'])+0*rprec (d_ATM_his ['lon']), dim1d='cell' )
            ATM_lon  = lmdz.geo2point ( 0*rprec (d_ATM_his ['lat'])+  rprec (d_ATM_his ['lon']), dim1d='cell' )
        ATM_aire = lmdz.geo2point ( rprec (d_ATM_his ['aire'][0]).squeeze(), cumul_poles=True, dim1d='cell' )
        ATM_fter = lmdz.geo2point ( rprec (d_ATM_his ['fract_ter'][0]), dim1d='cell' )
        ATM_foce = lmdz.geo2point ( rprec (d_ATM_his ['fract_oce'][0]), dim1d='cell' )
        ATM_fsic = lmdz.geo2point ( rprec (d_ATM_his ['fract_sic'][0]), dim1d='cell' )
        ATM_flic = lmdz.geo2point ( rprec (d_ATM_his ['fract_lic'][0]), dim1d='cell' )

    if ATM_HIS == 'ico' :
        echo ( 'ATM areas and fractions on ICO grid' )
        ATM_aire =  rprec (d_ATM_his ['aire']     [0]).squeeze()
        ATM_lat  =  rprec (d_ATM_his ['lat']         )
        ATM_lon  =  rprec (d_ATM_his ['lon']         )
        ATM_fter =  rprec (d_ATM_his ['fract_ter'][0])
        ATM_foce =  rprec (d_ATM_his ['fract_oce'][0])
        ATM_fsic =  rprec (d_ATM_his ['fract_sic'][0])
        ATM_flic =  rprec (d_ATM_his ['fract_lic'][0])

    if SRF :
        if SRF_HIS == 'latlon' :
            echo ( 'SRF areas and fractions on LATLON grid' )
            if 'lat_domain_landpoints_out' in d_SRF_his  :
                SRF_lat  = lmdz.geo2point (   rprec (d_SRF_his ['lat_domain_landpoints_out'])+0*rprec (d_SRF_his ['lon_domain_landpoints_out']), dim1d='cell' )
                SRF_lon  = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat_domain_landpoints_out'])+  rprec (d_SRF_his ['lon_domain_landpoints_out']), dim1d='cell' )
            else :
                if 'lat_domain_landpoints_out' in d_SRF_his :
                    SRF_lat  = lmdz.geo2point (   rprec (d_SRF_his ['lat_dom_out'])+0*rprec (d_SRF_his ['lon_dom_out']), dim1d='cell' )
                    SRF_lon  = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat_dom_out'])+  rprec (d_SRF_his ['lon_dom_out']), dim1d='cell' )
                else :
                    SRF_lat  = lmdz.geo2point (   rprec (d_SRF_his ['lat'])+0*rprec (d_SRF_his ['lon']), dim1d='cell' )
                    SRF_lon  = lmdz.geo2point ( 0*rprec (d_SRF_his ['lat'])+  rprec (d_SRF_his ['lon']), dim1d='cell' )

            SRF_areas     = lmdz.geo2point ( rprec (d_SRF_his ['Areas']   )   , dim1d='cell', cumul_poles=True )
            SRF_areafrac  = lmdz.geo2point ( rprec (d_SRF_his ['AreaFrac'])   , dim1d='cell', cumul_poles=True )
            SRF_contfrac  = lmdz.geo2point ( rprec (d_SRF_his ['Contfrac'])   , dim1d='cell', cumul_poles=True )
            SRF_aire = SRF_areafrac

        if SRF_HIS == 'ico' :
            echo ( 'SRF areas and fractions on ICO grid' )
            SRF_lat       =  rprec (d_SRF_his ['lat']     )
            SRF_lon       =  rprec (d_SRF_his ['lon']     )
            SRF_areas     =  rprec (d_SRF_his ['Areas']   )
            SRF_contfrac  =  rprec (d_SRF_his ['Contfrac'])
            SRF_aire      =  SRF_areas * SRF_contfrac

ATM_fsea      = ATM_foce + ATM_fsic
ATM_flnd      = ATM_fter + ATM_flic
ATM_aire_fter = ATM_aire * ATM_fter
ATM_aire_flic = ATM_aire * ATM_flic
ATM_aire_fsic = ATM_aire * ATM_fsic
ATM_aire_foce = ATM_aire * ATM_foce
ATM_aire_flnd = ATM_aire * ATM_flnd
ATM_aire_fsea = ATM_aire * ATM_fsea

#SRF_aire = SRF_aire.where ( np.abs (SRF_aire) < 1E15, 0. )

if ICO :
    # Area on icosahedron grid
    echo ( f'{file_DYN_aire = }' )
    d_DYN_aire = xr.open_dataset ( file_DYN_aire, decode_times=False ).squeeze()
    
    if SortIco :
            # Creation d'une clef de tri pour le fichier aire
            DYN_aire_keysort = np.lexsort ( (d_DYN_aire['lat'], d_DYN_aire['lon']) )
    else :
            DYN_aire_keysort = np.arange ( len ( d_DYN_aire['lat'] ) )
            
    DYN_lat = d_DYN_aire['lat']
    DYN_lon = d_DYN_aire['lon']
        
    DYN_aire = d_DYN_aire['aire']
    DYN_fsea = d_DYN_aire['fract_oce'] + d_DYN_aire['fract_sic']
        
    DYN_flnd = 1.0 - DYN_fsea
    DYN_fter = d_ATM_beg['FTER']
    DYN_flic = d_ATM_beg['FLIC']
    DYN_foce = d_ATM_beg['FOCE']
    DYN_aire_fter = DYN_aire * DYN_fter

#if ATM_HIS == 'ico' :
#    ATM_aire      = DYN_aire
#    ATM_aire_fter = DYN_aire * ATM_fter
#    ATM_aire_flic = DYN_aire * ATM_flic
#    ATM_aire_fsic = DYN_aire * ATM_fsic
#    ATM_aire_foce = DYN_aire * ATM_foce
#    ATM_aire_flnd = DYN_aire * ATM_flnd
#    ATM_aire_fsea = DYN_aire * ATM_fsea

if ATM_HIS == 'ico' :
    DYN_aire      = ATM_aire
    DYN_foce      = ATM_foce
    DYN_fsic      = ATM_fsic
    DYN_flic      = ATM_flic
    DYN_fter      = ATM_fter
    DYN_fsea      = ATM_fsea
    DYN_flnd      = ATM_flnd
    DYN_aire_fter = ATM_aire_fter
    DYN_aire_flic = ATM_aire_flic
    DYN_aire_fsic = ATM_aire_fsic
    DYN_aire_foce = ATM_aire_foce
    DYN_aire_flnd = ATM_aire_flnd
    DYN_aire_fsea = ATM_aire_fsea

if LMDZ :
    # Area on lon/lat grid
    DYN_aire = ATM_aire
    DYN_fsea = ATM_fsea
    DYN_flnd = ATM_flnd
    DYN_fter = rprec (d_ATM_beg['FTER'])
    DYN_flic = rprec (d_ATM_beg['FLIC'])
    DYN_aire_fter = DYN_aire * DYN_fter

if ICO and ATM_HIS == 'ico' :
    # Comparaison des aires sur ATM et DYN
    aire_diff = ATM_aire - DYN_aire
    echo ( 'f{Difference Aire hist file vs. grid file {aire_diff.mean()=} {aire_diff.min()=}  {aire_diff.max()=} ' )
    

# Functions computing integrals and sum
@Timer
def DYN_stock_int (stock) :
    '''Integrate (* surface) stock on atmosphere grid'''
    return wu.Psum ( (stock * DYN_aire).to_masked_array().ravel() )

@Timer
def ATM_flux_int (flux) :
    '''Integrate (* time * surface) flux on atmosphere grid'''
    return wu.Psum ( (flux * dtime_per_sec * ATM_aire).to_masked_array().ravel() )

@Timer
def LIC_flux_int (flux) :
    '''Integrate (* time * surface) flux on land ice grid'''
    return wu.Psum ( (flux * dtime_per_sec * ATM_aire_flic).to_masked_array().ravel() )

if SRF :
    @Timer
    def SRF_stock_int (stock) :
        '''Integrate (* surface) stock on land grid'''
        return wu.Ksum (  ( (stock * DYN_aire_fter).to_masked_array().ravel()) )

    @Timer
    def SRF_flux_int (flux) :
        '''Integrate (* time * surface) flux on land grid'''
        return wu.Psum (  (flux * dtime_per_sec * SRF_aire).to_masked_array().ravel() )

@Timer
def ONE_stock_int (stock) :
    '''Sum stock'''
    return wu.Psum ( stock.to_masked_array().ravel() )

@Timer
def ONE_flux_int (flux) :
    '''Integrate (* time) flux on area=1 grid'''
    return wu.Psum ( (flux * dtime_per_sec ).to_masked_array().ravel() )

@Timer
def Sprec ( tlist ) :
    '''Accurate sum of list of scalar elements'''
    return wu.Psum ( np.array ( tlist) )

ATM_aire_sea     = ATM_aire * ATM_fsea

ATM_aire_tot     = ONE_stock_int (ATM_aire)
ATM_aire_sea_tot = ONE_stock_int (ATM_aire_fsea)
ATM_aire_ter_tot = ONE_stock_int (ATM_aire_fter)
ATM_aire_lic_tot = ONE_stock_int (ATM_aire_flic)

DYN_aire_tot     = ONE_stock_int ( DYN_aire )
if SRF : SRF_aire_tot     = ONE_stock_int ( SRF_aire )

echo ('')
echo ( f'ATM DYN : Area of atmosphere        : {DYN_aire_tot:18.9e}' )
echo ( f'ATM HIS : Area of atmosphere        : {ATM_aire_tot:18.9e}' )
echo ( f'ATM HIS : Area of ter in atmosphere : {ATM_aire_ter_tot:18.9e}' )
echo ( f'ATM HIS : Area of lic in atmosphere : {ATM_aire_lic_tot:18.9e}' )
if SRF :
    echo ( f'ATM SRF : Area of atmosphere        : {SRF_aire_tot:18.9e}' )
echo ('')
echo ( 'ATM DYN : Area of atmosphere/(4pi R^2)        : {:18.9f}'.format(DYN_aire_tot/(RA*RA*4*np.pi) ) )
echo ( 'ATM HIS : Area of atmosphere/(4pi R^2)        : {:18.9f}'.format(ATM_aire_tot/(RA*RA*4*np.pi) ) )
echo ( 'ATM HIS : Area of ter in atmosphere/(4pi R^2) : {:18.9f}'.format(ATM_aire_ter_tot/(RA*RA*4*np.pi) ) )
if SRF :
    echo ( 'ATM SRF : Area of atmosphere/(4pi R^2)        : {:18.9f}'.format(SRF_aire_tot/(RA*RA*4*np.pi) ) )
    echo ('')
    echo ( f'ATM SRF : Area of atmosphere (no contfrac): {ONE_stock_int (SRF_areas):18.9e}' )


if np.abs (ATM_aire_tot/(RA*RA*4*np.pi) - 1.0) > 0.01 :
    raise RuntimeError ('Error of atmosphere surface interpolated on lon/lat grid')

echo ( '\n====================================================================================' )
echo ( f'-- ATM changes in stores  -- {Title} ' )

#-- Change in precipitable water from the atmosphere daily and monthly files
#-- Compute sum weighted by gridcell area (kg/m2) then convert to Sv

echo ( 'ATM vertical grid' )
ATM_Ahyb = d_ATM_his['Ahyb'].squeeze()
ATM_Bhyb = d_ATM_his['Bhyb'].squeeze()

echo ( 'Surface pressure' )
if ICO :
    DYN_psol_beg = d_DYN_beg['ps']
    DYN_psol_end = d_DYN_end['ps']
if LMDZ :
    DYN_psol_beg = lmdz.geo2point ( d_DYN_beg['ps'].isel(rlonv=slice(0,-1)), dim1d='cell' )
    DYN_psol_end = lmdz.geo2point ( d_DYN_end['ps'].isel(rlonv=slice(0,-1)), dim1d='cell' )

echo ( '3D Pressure at the interface layers (not scalar points)' )
DYN_pres_beg = ATM_Ahyb + ATM_Bhyb * DYN_psol_beg
DYN_pres_end = ATM_Ahyb + ATM_Bhyb * DYN_psol_end

echo ( 'Check computation of pressure levels' )

ind = np.empty (8)

ind[0] = (DYN_pres_beg[ 0]-DYN_psol_beg).min().item()
ind[1] = (DYN_pres_beg[ 0]-DYN_psol_beg).max().item()
ind[2] = (DYN_pres_beg[-1]).min().item()
ind[3] = (DYN_pres_beg[-1]).max().item()
ind[4] = (DYN_pres_end[ 0]-DYN_psol_end).min().item()
ind[5] = (DYN_pres_end[ 0]-DYN_psol_end).max().item()
ind[6] = (DYN_pres_end[-1]).min().item()
ind[7] = (DYN_pres_end[-1]).max().item()

if any ( ind != 0) :
    echo ( 'All values should be zero' )
    echo ( f'(DYN_pres_beg[ 0]-DYN_psol_beg).min().item() = {ind[0]}' )
    echo ( f'(DYN_pres_beg[ 0]-DYN_psol_beg).max().item() = {ind[1]}' )
    echo ( f'(DYN_pres_beg[-1]).min().item()              = {ind[2]}' )
    echo ( f'(DYN_pres_beg[-1]).max().item()              = {ind[3]}' )
    echo ( f'(DYN_pres_end[ 0]-DYN_psol_end).min().item() = {ind[4]}' )
    echo ( f'(DYN_pres_end[ 0]-DYN_psol_end).max().item() = {ind[5]}' )
    echo ( f'(DYN_pres_end[-1]).min().item()              = {ind[6]}' )
    echo ( f'(DYN_pres_end[-1]).max().item()              = {ind[7]}' )
    raise RuntimeError

klevp1 = ATM_Bhyb.shape[-1]
cell   = DYN_psol_beg.shape[-1]
klev   = klevp1 - 1

echo ( 'Layer thickness (pressure)' )
DYN_mass_beg = xr.DataArray ( np.empty( (klev, cell)), dims = ('sigs', 'cell'), coords=(np.arange(klev), np.arange(cell) ) )
DYN_mass_end = xr.DataArray ( np.empty( (klev, cell)), dims = ('sigs', 'cell'), coords=(np.arange(klev), np.arange(cell) ) )

for k in np.arange (klev) :
    DYN_mass_beg[k,:] = ( DYN_pres_beg[k,:] - DYN_pres_beg[k+1,:] ) / GRAV
    DYN_mass_end[k,:] = ( DYN_pres_end[k,:] - DYN_pres_end[k+1,:] ) / GRAV

DYN_mass_beg_2D = DYN_mass_beg.sum (dim='sigs')
DYN_mass_end_2D = DYN_mass_end.sum (dim='sigs')

DYN_mas_air_beg = DYN_stock_int ( DYN_mass_beg_2D )
DYN_mas_air_end = DYN_stock_int ( DYN_mass_end_2D )

echo ( 'Vertical and horizontal integral, and sum of liquid, solid and vapor water phases' )
if LMDZ :
    if 'H2Ov' in d_DYN_beg.variables :
        echo ('reading LATLON : H2Ov, H2Ol, H2Oi' )
        DYN_wat_beg = lmdz.geo3point ( d_DYN_beg['H2Ov'] + d_DYN_beg['H2Ol'] + d_DYN_beg['H2Oi'].isel(rlonv=slice(0,-1) ), dim1d='cell' )
        DYN_wat_end = lmdz.geo3point ( d_DYN_end['H2Ov'] + d_DYN_end['H2Ol'] + d_DYN_end['H2Oi'].isel(rlonv=slice(0,-1) ), dim1d='cell' )
    if 'H2Ov_g' in d_DYN_beg.variables :
        echo ('reading LATLON : H2O_g, H2O_l, H2O_s' )
        DYN_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).isel(rlonv=slice(0,-1) ), dim1d='cell' )
        DYN_wat_end = lmdz.geo3point ( (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).isel(rlonv=slice(0,-1) ), dim1d='cell' )
if ICO :
    if 'H2Ov_g' in d_DYN_beg.variables :
        echo ('reading ICO : H2Ov_g, H2Ov_l, H2Ov_s' )
        DYN_wat_beg = d_DYN_beg['H2Ov_g'] + d_DYN_beg['H2Ov_l'] + d_DYN_beg['H2Ov_s']
        DYN_wat_end = d_DYN_end['H2Ov_g'] + d_DYN_end['H2Ov_l'] + d_DYN_end['H2Ov_s']
    elif 'H2O_g' in d_DYN_beg.variables :
        echo ('reading ICO : H2O_g, H2O_l, H2O_s' )
        DYN_wat_beg = d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l']   + d_DYN_beg['H2O_s']
        DYN_wat_end = d_DYN_end['H2O_g'] + d_DYN_end['H2O_l']   + d_DYN_end['H2O_s']
    elif 'q' in d_DYN_beg.variables :
        echo ('reading ICO : q' )
        DYN_wat_beg = d_DYN_beg['q'].isel(nq=0) + d_DYN_beg['q'].isel(nq=1) + d_DYN_beg['q'].isel(nq=2)
        DYN_wat_end = d_DYN_end['q'].isel(nq=0) + d_DYN_end['q'].isel(nq=1) + d_DYN_end['q'].isel(nq=2)

if 'lev' in DYN_wat_beg.dims :
    DYN_wat_beg = DYN_wat_beg.rename ( {'lev':'sigs'} )
    DYN_wat_end = DYN_wat_end.rename ( {'lev':'sigs'} )

echo ( 'Compute water content : vertical and horizontal integral' )

DYN_wat_beg_2D =  (DYN_mass_beg * DYN_wat_beg).sum (dim='sigs')
DYN_wat_end_2D =  (DYN_mass_end * DYN_wat_end).sum (dim='sigs')

DYN_mas_wat_beg = DYN_stock_int ( DYN_wat_beg_2D )
DYN_mas_wat_end = DYN_stock_int ( DYN_wat_end_2D )

echo ( 'Variation of water content' )
dDYN_mas_wat = DYN_mas_wat_end - DYN_mas_wat_beg

echo ( f'\nChange of atmosphere water content (dynamics)  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'DYN_mas_air_beg = {:12.6e} kg | DYN_mas_air_end = {:12.6e} kg'.format (DYN_mas_air_beg, DYN_mas_air_end) )
echo ( 'DYN_mas_wat_beg = {:12.6e} kg | DYN_mas_wat_end = {:12.6e} kg'.format (DYN_mas_wat_beg, DYN_mas_wat_end) )
prtFlux ( 'dMass (atm)  ', dDYN_mas_wat, 'e', True )

ATM_sno_beg = d_ATM_beg['SNOW01']*d_ATM_beg['FTER'] + d_ATM_beg['SNOW02']*d_ATM_beg['FLIC'] + \
              d_ATM_beg['SNOW03']*d_ATM_beg['FOCE'] + d_ATM_beg['SNOW04']*d_ATM_beg['FSIC']
ATM_sno_end = d_ATM_end['SNOW01']*d_ATM_end['FTER'] + d_ATM_end['SNOW02']*d_ATM_end['FLIC'] + \
              d_ATM_end['SNOW03']*d_ATM_end['FOCE'] + d_ATM_end['SNOW04']*d_ATM_end['FSIC']

ATM_qs_beg  = d_ATM_beg['QS01']  *d_ATM_beg['FTER'] + d_ATM_beg['QS02']  *d_ATM_beg['FLIC'] + \
              d_ATM_beg['QS03']  *d_ATM_beg['FOCE'] + d_ATM_beg['QS04']  *d_ATM_beg['FSIC']
ATM_qs_end  = d_ATM_end['QS01']  *d_ATM_end['FTER'] + d_ATM_end['QS02']  *d_ATM_end['FLIC'] + \
              d_ATM_end['QS03']  *d_ATM_end['FOCE'] + d_ATM_end['QS04']  *d_ATM_end['FSIC']

ATM_qsol_beg     = d_ATM_beg['QSOL']
ATM_qsol_end     = d_ATM_end['QSOL']

LIC_sno_beg      = d_ATM_beg['SNOW02']*d_ATM_beg['FLIC']
LIC_sno_end      = d_ATM_end['SNOW02']*d_ATM_end['FLIC']

LIC_runlic0_beg  = d_ATM_beg['RUNOFFLIC0']
LIC_runlic0_end  = d_ATM_end['RUNOFFLIC0']

ATM_qs01_beg     = d_ATM_beg['QS01'] * d_ATM_beg['FTER']
ATM_qs02_beg     = d_ATM_beg['QS02'] * d_ATM_beg['FLIC']
ATM_qs03_beg     = d_ATM_beg['QS03'] * d_ATM_beg['FOCE']
ATM_qs04_beg     = d_ATM_beg['QS04'] * d_ATM_beg['FSIC']

ATM_qs01_end     = d_ATM_end['QS01'] * d_ATM_end['FTER']
ATM_qs02_end     = d_ATM_end['QS02'] * d_ATM_end['FLIC']
ATM_qs03_end     = d_ATM_end['QS03'] * d_ATM_end['FOCE']
ATM_qs04_end     = d_ATM_end['QS04'] * d_ATM_end['FSIC']

LIC_qs_beg = ATM_qs02_beg
LIC_qs_end = ATM_qs02_end

ATM_mas_sno_beg     = DYN_stock_int ( ATM_sno_beg  )
ATM_mas_sno_end     = DYN_stock_int ( ATM_sno_end  )

ATM_mas_qs_beg      = DYN_stock_int ( ATM_qs_beg   )
ATM_mas_qs_end      = DYN_stock_int ( ATM_qs_end   )
ATM_mas_qsol_beg    = DYN_stock_int ( ATM_qsol_beg )
ATM_mas_qs01_beg    = DYN_stock_int ( ATM_qs01_beg )
ATM_mas_qs02_beg    = DYN_stock_int ( ATM_qs02_beg )
ATM_mas_qs03_beg    = DYN_stock_int ( ATM_qs03_beg )
ATM_mas_qs04_beg    = DYN_stock_int ( ATM_qs04_beg )
ATM_mas_qsol_end    = DYN_stock_int ( ATM_qsol_end )
ATM_mas_qs01_end    = DYN_stock_int ( ATM_qs01_end )
ATM_mas_qs02_end    = DYN_stock_int ( ATM_qs02_end )
ATM_mas_qs03_end    = DYN_stock_int ( ATM_qs03_end )
ATM_mas_qs04_end    = DYN_stock_int ( ATM_qs04_end )

LIC_mas_sno_beg     = DYN_stock_int ( LIC_sno_beg     )
LIC_mas_sno_end     = DYN_stock_int ( LIC_sno_end     )
LIC_mas_runlic0_beg = DYN_stock_int ( LIC_runlic0_beg )
LIC_mas_runlic0_end = DYN_stock_int ( LIC_runlic0_end )

LIC_mas_qs_beg  = ATM_mas_qs02_beg
LIC_mas_qs_end  = ATM_mas_qs02_end
LIC_mas_wat_beg = LIC_mas_qs_beg + LIC_mas_sno_beg
LIC_mas_wat_end = LIC_mas_qs_end + LIC_mas_sno_end

dATM_mas_sno  = ATM_mas_sno_end  - ATM_mas_sno_beg
dATM_mas_qs   = ATM_mas_qs_end   - ATM_mas_qs_beg
dATM_mas_qsol = ATM_mas_qsol_end - ATM_mas_qsol_beg

dATM_mas_qs01 = ATM_mas_qs01_end - ATM_mas_qs01_beg
dATM_mas_qs02 = ATM_mas_qs02_end - ATM_mas_qs02_beg
dATM_mas_qs03 = ATM_mas_qs03_end - ATM_mas_qs03_beg
dATM_mas_qs04 = ATM_mas_qs04_end - ATM_mas_qs04_beg

dLIC_mas_qs      = LIC_mas_qs_end      - LIC_mas_qs_beg
dLIC_mas_sno     = LIC_mas_sno_end     - LIC_mas_sno_beg
dLIC_mas_runlic0 = LIC_mas_runlic0_end - LIC_mas_runlic0_beg

dLIC_mas_wat     = dLIC_mas_qs + dLIC_mas_sno # + dLIC_mas_runlic0

echo ( f'\nChange of atmosphere snow content (Land ice) -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'ATM_mas_sno_beg  = {:12.6e} kg | ATM_mas_sno_end  = {:12.6e} kg'.format (ATM_mas_sno_beg, ATM_mas_sno_end) )
prtFlux ( 'dMass (neige atm) ', dATM_mas_sno  , 'e', True  )

echo ( f'\nChange of soil humidity content -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'ATM_mas_qs_beg  = {:12.6e} kg | ATM_mas_qs_end  = {:12.6e} kg'.format (ATM_mas_qs_beg, ATM_mas_qs_end) )
prtFlux ( 'dMass (neige atm) ', dATM_mas_qs, 'e', True )

echo ( f'\nChange of atmosphere water+snow content -- {Title}  ' )
echo ( '------------------------------------------------------------------------------------' )
prtFlux ( 'dMass (eau + neige atm) ', dDYN_mas_wat + dATM_mas_sno , 'e', True)

if SRF :
    echo ( '\n====================================================================================' )
    echo ( f'-- SRF changes  -- {Title} ' )

    if Routing == 'SIMPLE' :
        RUN_mas_wat_fast_beg   = ONE_stock_int ( d_RUN_beg ['fast_reservoir']   )
        RUN_mas_wat_slow_beg   = ONE_stock_int ( d_RUN_beg ['slow_reservoir']   )
        RUN_mas_wat_stream_beg = ONE_stock_int ( d_RUN_beg ['stream_reservoir'] )
        RUN_mas_wat_flood_beg  = ONE_stock_int ( d_SRF_beg ['floodres']  )
        RUN_mas_wat_lake_beg   = ONE_stock_int ( d_SRF_beg ['lakeres']   )
        RUN_mas_wat_pond_beg   = ONE_stock_int ( d_SRF_beg ['pondres']   )

        RUN_mas_wat_fast_end   = ONE_stock_int ( d_RUN_end ['fast_reservoir']   )
        RUN_mas_wat_slow_end   = ONE_stock_int ( d_RUN_end ['slow_reservoir']   )
        RUN_mas_wat_stream_end = ONE_stock_int ( d_RUN_end ['stream_reservoir'] )

        RUN_mas_wat_flood_end  = ONE_stock_int ( d_SRF_end ['floodres']  )
        RUN_mas_wat_lake_end   = ONE_stock_int ( d_SRF_end ['lakeres']   )
        RUN_mas_wat_pond_end   = ONE_stock_int ( d_SRF_end ['pondres']   )

    if Routing == 'SECHIBA' :
        RUN_mas_wat_fast_beg   = ONE_stock_int ( d_SRF_beg ['fastres']   )
        RUN_mas_wat_slow_beg   = ONE_stock_int ( d_SRF_beg ['slowres']   )
        RUN_mas_wat_stream_beg = ONE_stock_int ( d_SRF_beg ['streamres'] )
        RUN_mas_wat_flood_beg  = ONE_stock_int ( d_SRF_beg ['floodres']  )
        RUN_mas_wat_lake_beg   = ONE_stock_int ( d_SRF_beg ['lakeres']   )
        RUN_mas_wat_pond_beg   = ONE_stock_int ( d_SRF_beg ['pondres']   )

        RUN_mas_wat_fast_end   = ONE_stock_int ( d_SRF_end ['fastres']   )
        RUN_mas_wat_slow_end   = ONE_stock_int ( d_SRF_end ['slowres']   )
        RUN_mas_wat_stream_end = ONE_stock_int ( d_SRF_end ['streamres'] )
        RUN_mas_wat_flood_end  = ONE_stock_int ( d_SRF_end ['floodres']  )
        RUN_mas_wat_lake_end   = ONE_stock_int ( d_SRF_end ['lakeres']   )
        RUN_mas_wat_pond_end   = ONE_stock_int ( d_SRF_end ['pondres']   )

    RUN_mas_wat_beg = Sprec ( [RUN_mas_wat_fast_beg , RUN_mas_wat_slow_beg, RUN_mas_wat_stream_beg,
                            RUN_mas_wat_flood_beg, RUN_mas_wat_lake_beg, RUN_mas_wat_pond_beg] )

    RUN_mas_wat_end = Sprec ( [RUN_mas_wat_fast_end  , RUN_mas_wat_slow_end , RUN_mas_wat_stream_end,
                            RUN_mas_wat_flood_end , RUN_mas_wat_lake_end , RUN_mas_wat_pond_end] )

    dRUN_mas_wat_fast   = RUN_mas_wat_fast_end   - RUN_mas_wat_fast_beg
    dRUN_mas_wat_slow   = RUN_mas_wat_slow_end   - RUN_mas_wat_slow_beg
    dRUN_mas_wat_stream = RUN_mas_wat_stream_end - RUN_mas_wat_stream_beg
    dRUN_mas_wat_flood  = RUN_mas_wat_flood_end  - RUN_mas_wat_flood_beg
    dRUN_mas_wat_lake   = RUN_mas_wat_lake_end   - RUN_mas_wat_lake_beg
    dRUN_mas_wat_pond   = RUN_mas_wat_pond_end   - RUN_mas_wat_pond_beg

    dRUN_mas_wat        = RUN_mas_wat_end        - RUN_mas_wat_beg

    echo ( f'\nRunoff reservoirs -- {Title} ')
    echo (  '------------------------------------------------------------------------------------' )
    echo ( f'RUN_mas_wat_fast_beg   = {RUN_mas_wat_fast_beg  :12.6e} kg | RUN_mas_wat_fast_end   = {RUN_mas_wat_fast_end  :12.6e} kg ' )
    echo ( f'RUN_mas_wat_slow_beg   = {RUN_mas_wat_slow_beg  :12.6e} kg | RUN_mas_wat_slow_end   = {RUN_mas_wat_slow_end  :12.6e} kg ' )
    echo ( f'RUN_mas_wat_stream_beg = {RUN_mas_wat_stream_beg:12.6e} kg | RUN_mas_wat_stream_end = {RUN_mas_wat_stream_end:12.6e} kg ' )
    echo ( f'RUN_mas_wat_flood_beg  = {RUN_mas_wat_flood_beg :12.6e} kg | RUN_mas_wat_flood_end  = {RUN_mas_wat_flood_end :12.6e} kg ' )
    echo ( f'RUN_mas_wat_lake_beg   = {RUN_mas_wat_lake_beg  :12.6e} kg | RUN_mas_wat_lake_end   = {RUN_mas_wat_lake_end  :12.6e} kg ' )
    echo ( f'RUN_mas_wat_pond_beg   = {RUN_mas_wat_pond_beg  :12.6e} kg | RUN_mas_wat_pond_end   = {RUN_mas_wat_pond_end  :12.6e} kg ' )
    echo ( f'RUN_mas_wat_beg        = {RUN_mas_wat_beg       :12.6e} kg | RUN_mas_wat_end        = {RUN_mas_wat_end       :12.6e} kg ' )

    echo ( '------------------------------------------------------------------------------------' )

    prtFlux ( 'dMass (fast)   ', dRUN_mas_wat_fast  , 'e', True )
    prtFlux ( 'dMass (slow)   ', dRUN_mas_wat_slow  , 'e', True )
    prtFlux ( 'dMass (stream) ', dRUN_mas_wat_stream, 'e', True )
    prtFlux ( 'dMass (flood)  ', dRUN_mas_wat_flood , 'e', True )
    prtFlux ( 'dMass (lake)   ', dRUN_mas_wat_lake  , 'e', True )
    prtFlux ( 'dMass (pond)   ', dRUN_mas_wat_pond  , 'e', True )
    prtFlux ( 'dMass (all)    ', dRUN_mas_wat       , 'e', True )

    echo ( f'\nWater content in routing  -- {Title} ' )
    echo ( '------------------------------------------------------------------------------------' )
    echo ( f'RUN_mas_wat_beg = {RUN_mas_wat_end:12.6e} kg | RUN_mas_wat_end = {RUN_mas_wat_end:12.6e} kg' )
    prtFlux ( 'dMass (routing) ', dRUN_mas_wat , 'e', True   )

    echo ( '\n====================================================================================' )
    print ( 'Reading SRF restart')
    SRF_tot_watveg_beg  = d_SRF_beg['tot_watveg_beg']  ; SRF_tot_watveg_beg  = SRF_tot_watveg_beg .where (SRF_tot_watveg_beg  < 1E15, 0.)
    SRF_tot_watsoil_beg = d_SRF_beg['tot_watsoil_beg'] ; SRF_tot_watsoil_beg = SRF_tot_watsoil_beg.where (SRF_tot_watsoil_beg < 1E15, 0.)
    SRF_snow_beg        = d_SRF_beg['snow_beg']        ; SRF_snow_beg        = SRF_snow_beg       .where (SRF_snow_beg        < 1E15, 0.)
    SRF_lakeres_beg     = d_SRF_beg['lakeres']         ; SRF_lakeres_beg     = SRF_lakeres_beg    .where (SRF_lakeres_beg     < 1E15, 0.)

    SRF_tot_watveg_end  = d_SRF_end['tot_watveg_beg']  ; SRF_tot_watveg_end  = SRF_tot_watveg_end .where (SRF_tot_watveg_end  < 1E15, 0.)
    SRF_tot_watsoil_end = d_SRF_end['tot_watsoil_beg'] ; SRF_tot_watsoil_end = SRF_tot_watsoil_end.where (SRF_tot_watsoil_end < 1E15, 0.)
    SRF_snow_end        = d_SRF_end['snow_beg']        ; SRF_snow_end        = SRF_snow_end       .where (SRF_snow_end        < 1E15, 0.)
    SRF_lakeres_end     = d_SRF_end['lakeres']         ; SRF_lakeres_end     = SRF_lakeres_end    .where (SRF_lakeres_end     < 1E15, 0.)

    if LMDZ :
        SRF_tot_watveg_beg  = lmdz.geo2point (SRF_tot_watveg_beg , dim1d='cell')
        SRF_tot_watsoil_beg = lmdz.geo2point (SRF_tot_watsoil_beg, dim1d='cell')
        SRF_snow_beg        = lmdz.geo2point (SRF_snow_beg       , dim1d='cell')
        SRF_lakeres_beg     = lmdz.geo2point (SRF_lakeres_beg    , dim1d='cell')
        SRF_tot_watveg_end  = lmdz.geo2point (SRF_tot_watveg_end , dim1d='cell')
        SRF_tot_watsoil_end = lmdz.geo2point (SRF_tot_watsoil_end, dim1d='cell')
        SRF_snow_end        = lmdz.geo2point (SRF_snow_end       , dim1d='cell')
        SRF_lakeres_end     = lmdz.geo2point (SRF_lakeres_end    , dim1d='cell')


    # Stock dSoilHum dInterce dSWE dStream dFastR dSlowR dLake dPond dFlood

    SRF_wat_beg = SRF_tot_watveg_beg + SRF_tot_watsoil_beg + SRF_snow_beg
    SRF_wat_end = SRF_tot_watveg_end + SRF_tot_watsoil_end + SRF_snow_end

    echo ( '\n====================================================================================' )
    print ('Computing integrals')

    print ( ' 1/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_watveg_beg   = SRF_stock_int ( SRF_tot_watveg_beg  )
    print ( ' 2/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_watsoil_beg  = SRF_stock_int ( SRF_tot_watsoil_beg )
    print ( ' 3/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_snow_beg     = SRF_stock_int ( SRF_snow_beg        )
    print ( ' 4/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_lake_beg     = ONE_stock_int ( SRF_lakeres_beg     )
    print ( ' 5/8', end='' ) ; sys.stdout.flush ()

    SRF_mas_watveg_end   = SRF_stock_int ( SRF_tot_watveg_end  )
    print ( ' 6/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_watsoil_end  = SRF_stock_int ( SRF_tot_watsoil_end )
    print ( ' 7/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_snow_end     = SRF_stock_int ( SRF_snow_end        )
    print ( ' 8/8', end='' ) ; sys.stdout.flush ()
    SRF_mas_lake_end     = ONE_stock_int ( SRF_lakeres_end     )

    print (' -- ') ; sys.stdout.flush ()

    dSRF_mas_watveg   = Sprec ( [SRF_mas_watveg_end , -SRF_mas_watveg_beg] )
    dSRF_mas_watsoil  = Sprec ( [SRF_mas_watsoil_end, -SRF_mas_watsoil_beg] )
    dSRF_mas_snow     = Sprec ( [SRF_mas_snow_end   , -SRF_mas_snow_beg] )
    dSRF_mas_lake     = Sprec ( [SRF_mas_lake_end   , -SRF_mas_lake_beg] )

    echo ( '------------------------------------------------------------------------------------' )
    echo ( f'\nSurface reservoirs  -- {Title} ')
    echo ( f'SRF_mas_watveg_beg   = {SRF_mas_watveg_beg :12.6e} kg | SRF_mas_watveg_end   = {SRF_mas_watveg_end :12.6e} kg ' )
    echo ( f'SRF_mas_watsoil_beg  = {SRF_mas_watsoil_beg:12.6e} kg | SRF_mas_watsoil_end  = {SRF_mas_watsoil_end:12.6e} kg ' )
    echo ( f'SRF_mas_snow_beg     = {SRF_mas_snow_beg   :12.6e} kg | SRF_mas_snow_end     = {SRF_mas_snow_end   :12.6e} kg ' )
    echo ( f'SRF_mas_lake_beg     = {SRF_mas_lake_beg   :12.6e} kg | SRF_mas_lake_end     = {SRF_mas_lake_end   :12.6e} kg ' )

    prtFlux ( 'dMass (watveg) ', dSRF_mas_watveg , 'e' , True )
    prtFlux ( 'dMass (watsoil)', dSRF_mas_watsoil, 'e' , True )
    prtFlux ( 'dMass (snow)   ', dSRF_mas_snow   , 'e' , True )
    prtFlux ( 'dMass (lake)   ', dSRF_mas_lake   , 'e' , True )

    SRF_mas_wat_beg = Sprec ( [SRF_mas_watveg_beg , SRF_mas_watsoil_beg, SRF_mas_snow_beg] )
    SRF_mas_wat_end = Sprec ( [SRF_mas_watveg_end , SRF_mas_watsoil_end, SRF_mas_snow_end] )

    dSRF_mas_wat    = Sprec ( [+SRF_mas_watveg_end , +SRF_mas_watsoil_end, +SRF_mas_snow_end,
                               -SRF_mas_watveg_beg , -SRF_mas_watsoil_beg, -SRF_mas_snow_beg]  )

    echo ( '------------------------------------------------------------------------------------' )
    echo ( f'Water content in surface  -- {Title} ' )
    echo ( f'SRF_mas_wat_beg   = {SRF_mas_wat_beg:12.6e} kg | SRF_mas_wat_end  = {SRF_mas_wat_end:12.6e} kg ')
    prtFlux ( 'dMass (water srf)', dSRF_mas_wat, 'e', True )

    echo ( '------------------------------------------------------------------------------------' )
    echo ( 'Water content in  ATM + SRF + RUN + LAKE' )
    echo ( 'mas_wat_beg = {:12.6e} kg | mas_wat_end = {:12.6e} kg '.
            format (DYN_mas_wat_beg + ATM_mas_sno_beg + RUN_mas_wat_beg + SRF_mas_wat_beg + SRF_mas_lake_beg ,
                    DYN_mas_wat_end + ATM_mas_sno_end + RUN_mas_wat_end + SRF_mas_wat_end + SRF_mas_lake_end ) )
    prtFlux ( 'dMass (water atm+srf+run+lake)', dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat + dSRF_mas_lake, 'e', True)

echo ( '\n====================================================================================' )
echo ( f'-- ATM Fluxes  -- {Title} ' )

if ATM_HIS == 'latlon' :
    echo ( ' latlon case' )
    ATM_wbilo_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_oce']), dim1d='cell' )
    ATM_wbilo_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_sic']), dim1d='cell' )
    ATM_wbilo_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_ter']), dim1d='cell' )
    ATM_wbilo_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wbilo_lic']), dim1d='cell' )
    ATM_runofflic   = lmdz.geo2point ( rprec (d_ATM_his ['runofflic']), dim1d='cell' )
    ATM_fqcalving   = lmdz.geo2point ( rprec (d_ATM_his ['fqcalving']), dim1d='cell' )
    ATM_fqfonte     = lmdz.geo2point ( rprec (d_ATM_his ['fqfonte']  ), dim1d='cell' )
    ATM_precip      = lmdz.geo2point ( rprec (d_ATM_his ['precip']   ), dim1d='cell' )
    ATM_snowf       = lmdz.geo2point ( rprec (d_ATM_his ['snow']     ), dim1d='cell' )
    ATM_evap        = lmdz.geo2point ( rprec (d_ATM_his ['evap']     ), dim1d='cell' )
    ATM_wevap_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_ter']), dim1d='cell' )
    ATM_wevap_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_oce']), dim1d='cell' )
    ATM_wevap_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_lic']), dim1d='cell' )
    ATM_wevap_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wevap_sic']), dim1d='cell' )
    ATM_wrain_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_ter']), dim1d='cell' )
    ATM_wrain_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_oce']), dim1d='cell' )
    ATM_wrain_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_lic']), dim1d='cell' )
    ATM_wrain_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wrain_sic']), dim1d='cell' )
    ATM_wsnow_ter   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_ter']), dim1d='cell' )
    ATM_wsnow_oce   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_oce']), dim1d='cell' )
    ATM_wsnow_lic   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_lic']), dim1d='cell' )
    ATM_wsnow_sic   = lmdz.geo2point ( rprec (d_ATM_his ['wsnow_sic']), dim1d='cell' )
    ATM_runofflic   = lmdz.geo2point ( rprec (d_ATM_his ['runofflic']), dim1d='cell' )
    echo ( 'End of LATLON case')

if ATM_HIS == 'ico' :
    echo (' ico case')
    ATM_wbilo_oce   = rprec (d_ATM_his ['wbilo_oce'])
    ATM_wbilo_sic   = rprec (d_ATM_his ['wbilo_sic'])
    ATM_wbilo_ter   = rprec (d_ATM_his ['wbilo_ter'])
    ATM_wbilo_lic   = rprec (d_ATM_his ['wbilo_lic'])
    ATM_runofflic   = rprec (d_ATM_his ['runofflic'])
    ATM_fqcalving   = rprec (d_ATM_his ['fqcalving'])
    ATM_fqfonte     = rprec (d_ATM_his ['fqfonte']  )
    ATM_precip      = rprec (d_ATM_his ['precip']   )
    ATM_snowf       = rprec (d_ATM_his ['snow']     )
    ATM_evap        = rprec (d_ATM_his ['evap']     )
    ATM_wevap_ter   = rprec (d_ATM_his ['wevap_ter'])
    ATM_wevap_oce   = rprec (d_ATM_his ['wevap_oce'])
    ATM_wevap_lic   = rprec (d_ATM_his ['wevap_lic'])
    ATM_wevap_sic   = rprec (d_ATM_his ['wevap_sic'])
    ATM_runofflic   = rprec (d_ATM_his ['runofflic'])
    ATM_wevap_ter   = rprec (d_ATM_his ['wevap_ter'])
    ATM_wevap_oce   = rprec (d_ATM_his ['wevap_oce'])
    ATM_wevap_lic   = rprec (d_ATM_his ['wevap_lic'])
    ATM_wevap_sic   = rprec (d_ATM_his ['wevap_sic'])
    ATM_wrain_ter   = rprec (d_ATM_his ['wrain_ter'])
    ATM_wrain_oce   = rprec (d_ATM_his ['wrain_oce'])
    ATM_wrain_lic   = rprec (d_ATM_his ['wrain_lic'])
    ATM_wrain_sic   = rprec (d_ATM_his ['wrain_sic'])
    ATM_wsnow_ter   = rprec (d_ATM_his ['wsnow_ter'])
    ATM_wsnow_oce   = rprec (d_ATM_his ['wsnow_oce'])
    ATM_wsnow_lic   = rprec (d_ATM_his ['wsnow_lic'])
    ATM_wsnow_sic   = rprec (d_ATM_his ['wsnow_sic'])
    echo ( 'End of ico case ')

echo ( 'ATM wprecip_oce' )
ATM_wprecip_oce = ATM_wrain_oce + ATM_wsnow_oce
ATM_wprecip_ter = ATM_wrain_ter + ATM_wsnow_ter
ATM_wprecip_sic = ATM_wrain_sic + ATM_wsnow_sic
ATM_wprecip_lic = ATM_wrain_lic + ATM_wsnow_lic

ATM_wbilo       = ATM_wbilo_oce   + ATM_wbilo_sic   + ATM_wbilo_ter   + ATM_wbilo_lic
ATM_wevap       = ATM_wevap_oce   + ATM_wevap_sic   + ATM_wevap_ter   + ATM_wevap_lic
ATM_wprecip     = ATM_wprecip_oce + ATM_wprecip_sic + ATM_wprecip_ter + ATM_wprecip_lic
ATM_wsnow       = ATM_wsnow_oce   + ATM_wsnow_sic   + ATM_wsnow_ter   + ATM_wsnow_lic
ATM_wrain       = ATM_wrain_oce   + ATM_wrain_sic   + ATM_wrain_ter   + ATM_wrain_lic
ATM_wemp        = ATM_wevap - ATM_wprecip
ATM_emp         = ATM_evap  - ATM_precip

ATM_wprecip_sea = ATM_wprecip_oce + ATM_wprecip_sic
ATM_wsnow_sea   = ATM_wsnow_oce   + ATM_wsnow_sic
ATM_wrain_sea   = ATM_wrain_oce   + ATM_wrain_sic
ATM_wbilo_sea   = ATM_wbilo_oce   + ATM_wbilo_sic
ATM_wevap_sea   = ATM_wevap_sic   + ATM_wevap_oce

ATM_wemp_ter    = ATM_wevap_ter - ATM_wprecip_ter
ATM_wemp_oce    = ATM_wevap_oce - ATM_wprecip_oce
ATM_wemp_sic    = ATM_wevap_sic - ATM_wprecip_sic
ATM_wemp_lic    = ATM_wevap_lic - ATM_wprecip_lic
ATM_wemp_sea    = ATM_wevap_sic - ATM_wprecip_oce

if SRF :
    if RUN_HIS == 'latlon' :
        echo ( 'RUN costalflow Grille LATLON' )
        if TestInterp :
            echo ( 'RUN runoff TestInterp' )
            RUN_runoff      = lmdz.geo2point ( rprec (d_RUN_his ['runoff_contfrac_interp']  )   , dim1d='cell' )
            RUN_drainage    = lmdz.geo2point ( rprec (d_RUN_his ['drainage_contfrac_interp'])   , dim1d='cell' )
        else :
            echo ( 'RUN runoff' )
            RUN_runoff      = lmdz.geo2point ( rprec (d_RUN_his ['runoff']         ), dim1d='cell' )
            RUN_drainage    = lmdz.geo2point ( rprec (d_RUN_his ['drainage']       ), dim1d='cell' )

        RUN_coastalflow     = lmdz.geo2point ( rprec (d_RUN_his ['coastalflow']    ), dim1d='cell' )
        RUN_riverflow       = lmdz.geo2point ( rprec (d_RUN_his ['riverflow']      ), dim1d='cell' )
        RUN_riversret       = lmdz.geo2point ( rprec (d_RUN_his ['riversret']      ), dim1d='cell' )
        RUN_coastalflow_cpl = lmdz.geo2point ( rprec (d_RUN_his ['coastalflow_cpl']), dim1d='cell' )
        RUN_riverflow_cpl   = lmdz.geo2point ( rprec (d_RUN_his ['riverflow_cpl']  ), dim1d='cell' )

    if RUN_HIS == 'ico' :
        echo ( 'RUN costalflow Grille ICO' )
        RUN_coastalflow =  rprec (d_RUN_his ['coastalflow'])
        RUN_riverflow   =  rprec (d_RUN_his ['riverflow']  )
        RUN_runoff      =  rprec (d_RUN_his ['runoff']     )
        RUN_drainage    =  rprec (d_RUN_his ['drainage']   )
        RUN_riversret   =  rprec (d_RUN_his ['riversret']  )

        RUN_coastalflow_cpl = rprec (d_RUN_his ['coastalflow_cpl'])
        RUN_riverflow_cpl   = rprec (d_RUN_his ['riverflow_cpl']  )

    Step = 0

    if SRF_HIS == 'latlon' :
        if TestInterp :
            echo ( 'SRF rain TestInterp' )
            SRF_rain     = lmdz.geo2point ( rprec (d_SRF_his ['rain_contfrac_interp'] ), dim1d='cell')
            SRF_evap     = lmdz.geo2point ( rprec (d_SRF_his ['evap_contfrac_interp'] ), dim1d='cell')
            SRF_snowf    = lmdz.geo2point ( rprec (d_SRF_his ['snow_contfrac_interp'] ), dim1d='cell')
            SRF_subli    = lmdz.geo2point ( rprec (d_SRF_his ['subli_contfrac_interp']), dim1d='cell')
            SRF_transpir = lmdz.geo2point ( rprec (d_SRF_his ['transpir_contfrac_interp']).sum(dim='veget'), dim1d='cell' )
            #SRF_rain.attrs.update     ( d_SRF_his ['rain_contfrac_interp'].attrs )
            #SRF_evap.attrs.update     ( d_SRF_his ['evap_contfrac_interp'].attrs )
            #SRF_snowf.attrs.update    ( d_SRF_his ['snow_contfrac_interp'].attrs )
            #SRF_subli.attrs.update    ( d_SRF_his ['subli_contfrac_interp'].attrs )
            #SRF_transpir.attrs.update ( d_SRF_his ['transpir_contfrac_interp'].attrs )
        else :
            echo ( 'SRF rain' )
            SRF_rain     = lmdz.geo2point ( rprec (d_SRF_his ['rain'] ) , dim1d='cell')
            SRF_evap     = lmdz.geo2point ( rprec (d_SRF_his ['evap'] ) , dim1d='cell')
            SRF_snowf    = lmdz.geo2point ( rprec (d_SRF_his ['snowf']) , dim1d='cell')
            SRF_subli    = lmdz.geo2point ( rprec (d_SRF_his ['subli']) , dim1d='cell')
            SRF_transpir = lmdz.geo2point ( rprec (d_SRF_his ['transpir']).sum(dim='veget'), dim1d='cell' )

    if SRF_HIS == 'ico' :
        echo ( 'SRF rain')
        SRF_rain     = rprec (d_SRF_his ['rain'] )
        SRF_evap     = rprec (d_SRF_his ['evap'] )
        SRF_snowf    = rprec (d_SRF_his ['snowf'])
        SRF_subli    = rprec (d_SRF_his ['subli'])
        SRF_transpir = rprec (d_SRF_his ['transpir']).sum(dim='veget')

    echo ( 'SRF emp' )
    SRF_transpir.attrs['units'] = d_SRF_his ['transpir'].attrs['units']
    SRF_emp      = SRF_evap - SRF_rain - SRF_snowf ; SRF_emp.attrs['units'] = SRF_rain.attrs['units']

    ## Correcting units of SECHIBA variables
    def mmd2si ( pvar ) :
        '''Change unit from mm/d or m^3/s to kg/s if needed'''
        if 'units' in pvar.attrs :
            if pvar.attrs['units'] in ['m^3/s', 'm3/s', 'm3.s-1',] :
                pvar.values = pvar.values  * ATM_RHO                    ;  pvar.attrs['units'] = 'kg/s'
            if pvar.attrs['units'] == 'mm/d' :
                pvar.values = pvar.values  * ATM_RHO * (1e-3/lmdz.RDAY) ;  pvar.attrs['units'] = 'kg/s'
            if pvar.attrs['units'] in ['m^3', 'm3', ] :
                pvar.values = pvar.values  * ATM_RHO                    ;  pvar.attrs['units'] = 'kg'

    for var in [ 'runoff', 'drainage', 'riversret', 'coastalflow', 'riverflow', 'coastalflow_cpl', 'riverflow_cpl' ] :
        zvar = locals()['RUN_' + var]
        mmd2si (zvar)

    for var in ['evap', 'snowf', 'subli', 'transpir', 'rain', 'emp' ] :
        zvar = locals()['SRF_' + var]
        mmd2si (zvar)

    echo ( 'RUN input' )
    RUN_input  = RUN_runoff      + RUN_drainage
    RUN_output = RUN_coastalflow + RUN_riverflow

echo ( 'ATM flw_wbilo' )
ATM_flx_wbilo       = ATM_flux_int ( ATM_wbilo      )
ATM_flx_wevap       = ATM_flux_int ( ATM_wevap      )
ATM_flx_wprecip     = ATM_flux_int ( ATM_wprecip    )
ATM_flx_wsnow       = ATM_flux_int ( ATM_wsnow      )
ATM_flx_wrain       = ATM_flux_int ( ATM_wrain      )
ATM_flx_wemp        = ATM_flux_int ( ATM_wemp       )

ATM_flx_wbilo_lic   = ATM_flux_int ( ATM_wbilo_lic  )
ATM_flx_wbilo_oce   = ATM_flux_int ( ATM_wbilo_oce  )
ATM_flx_wbilo_sea   = ATM_flux_int ( ATM_wbilo_sea  )
ATM_flx_wbilo_sic   = ATM_flux_int ( ATM_wbilo_sic  )
ATM_flx_wbilo_ter   = ATM_flux_int ( ATM_wbilo_ter  )
# Type d'integration a verifier
ATM_flx_calving     = ATM_flux_int ( ATM_fqcalving  )
ATM_flx_fqfonte     = ATM_flux_int ( ATM_fqfonte    )

LIC_flx_calving     = LIC_flux_int ( ATM_fqcalving  )
LIC_flx_fqfonte     = LIC_flux_int ( ATM_fqfonte    )

echo ( 'ATM flx precip' )
ATM_flx_precip      = ATM_flux_int ( ATM_precip     )
ATM_flx_snowf       = ATM_flux_int ( ATM_snowf      )
ATM_flx_evap        = ATM_flux_int ( ATM_evap       )
ATM_flx_runlic      = ATM_flux_int ( ATM_runofflic  )

LIC_flx_precip      = LIC_flux_int ( ATM_precip     )
LIC_flx_snowf       = LIC_flux_int ( ATM_snowf      )
LIC_flx_evap        = LIC_flux_int ( ATM_evap       )
LIC_flx_runlic      = LIC_flux_int ( ATM_runofflic  )

echo ( 'ATM flx_wrain_ter' )
ATM_flx_wrain_ter    = ATM_flux_int ( ATM_wrain_ter )
ATM_flx_wrain_oce    = ATM_flux_int ( ATM_wrain_oce )
ATM_flx_wrain_lic    = ATM_flux_int ( ATM_wrain_lic )
ATM_flx_wrain_sic    = ATM_flux_int ( ATM_wrain_sic )
ATM_flx_wrain_sea    = ATM_flux_int ( ATM_wrain_sea )

ATM_flx_wsnow_ter    = ATM_flux_int ( ATM_wsnow_ter )
ATM_flx_wsnow_oce    = ATM_flux_int ( ATM_wsnow_oce )
ATM_flx_wsnow_lic    = ATM_flux_int ( ATM_wsnow_lic )
ATM_flx_wsnow_sic    = ATM_flux_int ( ATM_wsnow_sic )
ATM_flx_wsnow_sea    = ATM_flux_int ( ATM_wsnow_sea )

echo ( 'ATM flx_evap_ter' )
ATM_flx_wevap_ter    = ATM_flux_int ( ATM_wevap_ter )
ATM_flx_wevap_oce    = ATM_flux_int ( ATM_wevap_oce )
ATM_flx_wevap_lic    = ATM_flux_int ( ATM_wevap_lic )
ATM_flx_wevap_sic    = ATM_flux_int ( ATM_wevap_sic )
ATM_flx_wevap_sea    = ATM_flux_int ( ATM_wevap_sea )
ATM_flx_wprecip_lic  = ATM_flux_int ( ATM_wprecip_lic )
ATM_flx_wprecip_oce  = ATM_flux_int ( ATM_wprecip_oce )
ATM_flx_wprecip_sic  = ATM_flux_int ( ATM_wprecip_sic )
ATM_flx_wprecip_ter  = ATM_flux_int ( ATM_wprecip_ter )
ATM_flx_wprecip_sea  = ATM_flux_int ( ATM_wprecip_sea )
ATM_flx_wemp_lic     = ATM_flux_int ( ATM_wemp_lic )
ATM_flx_wemp_oce     = ATM_flux_int ( ATM_wemp_oce )
ATM_flx_wemp_sic     = ATM_flux_int ( ATM_wemp_sic )
ATM_flx_wemp_ter     = ATM_flux_int ( ATM_wemp_ter )
ATM_flx_wemp_sea     = ATM_flux_int ( ATM_wemp_sea )

ATM_flx_emp          = ATM_flux_int ( ATM_emp )

if SRF :
    echo ( 'RUN flx_coastal' )
    RUN_flx_coastal     = ONE_flux_int ( RUN_coastalflow)
    echo ( 'RUN flx_river' )
    RUN_flx_river       = ONE_flux_int ( RUN_riverflow  )
    echo ( 'RUN flx_coastal_cpl' )
    RUN_flx_coastal_cpl = ONE_flux_int ( RUN_coastalflow_cpl)
    echo ( 'RUN flx_river_cpl' )
    RUN_flx_river_cpl   = ONE_flux_int ( RUN_riverflow_cpl  )
    echo ( 'RUN flx_drainage' )
    RUN_flx_drainage    = SRF_flux_int ( RUN_drainage   )
    echo ( 'RUN flx_riversset' )
    RUN_flx_riversret   = SRF_flux_int ( RUN_riversret  )
    echo ( 'RUN flx_runoff' )
    RUN_flx_runoff      = SRF_flux_int ( RUN_runoff     )
    echo ( 'RUN flx_input' )
    RUN_flx_input       = SRF_flux_int ( RUN_input      )
    echo ( 'RUN flx_output' )
    RUN_flx_output      = ONE_flux_int ( RUN_output     )

    echo ( 'RUN flx_bil' ) ; Step += 1
    #RUN_flx_bil    = RUN_flx_input   - RUN_flx_output
    #RUN_flx_rivcoa = RUN_flx_coastal + RUN_flx_river

    RUN_flx_bil    = ONE_flux_int ( RUN_input       - RUN_output)
    RUN_flx_rivcoa = ONE_flux_int ( RUN_coastalflow + RUN_riverflow)

prtFlux ('wbilo_oce            ', ATM_flx_wbilo_oce     , 'f' )
prtFlux ('wbilo_sic            ', ATM_flx_wbilo_sic     , 'f' )
prtFlux ('wbilo_sic+oce        ', ATM_flx_wbilo_sea     , 'f' )
prtFlux ('wbilo_ter            ', ATM_flx_wbilo_ter     , 'f' )
prtFlux ('wbilo_lic            ', ATM_flx_wbilo_lic     , 'f' )
prtFlux ('Sum wbilo_*          ', ATM_flx_wbilo         , 'f', True)
prtFlux ('E-P                  ', ATM_flx_emp           , 'f', True)
prtFlux ('calving              ', ATM_flx_calving       , 'f' )
prtFlux ('fqfonte              ', ATM_flx_fqfonte       , 'f' )
prtFlux ('precip               ', ATM_flx_precip        , 'f' )
prtFlux ('snowf                ', ATM_flx_snowf         , 'f' )
prtFlux ('evap                 ', ATM_flx_evap          , 'f' )
prtFlux ('runoff lic           ', ATM_flx_runlic        , 'f' )

prtFlux ('ATM_flx_wevap*       ', ATM_flx_wevap         , 'f' )
prtFlux ('ATM_flx_wrain*       ', ATM_flx_wrain         , 'f' )
prtFlux ('ATM_flx_wsnow*       ', ATM_flx_wsnow         , 'f' )
prtFlux ('ATM_flx_wprecip*     ', ATM_flx_wprecip       , 'f' )
prtFlux ('ATM_flx_wemp*        ', ATM_flx_wemp          , 'f', True )

echo ( 'Errors <field> vs. wbil_<field>' )
prtFlux ('ERROR evap           ', ATM_flx_wevap   - ATM_flx_evap  , 'e', True )
prtFlux ('ERROR precip         ', ATM_flx_wprecip - ATM_flx_precip, 'e', True )
prtFlux ('ERROR snow           ', ATM_flx_wsnow   - ATM_flx_snowf , 'e', True )
prtFlux ('ERROR emp            ', ATM_flx_wemp    - ATM_flx_emp   , 'e', True )

if SRF :
    echo ( '\n====================================================================================' )
    echo ( f'-- RUNOFF Fluxes  -- {Title} ' )
    prtFlux ('coastalflow   ', RUN_flx_coastal    , 'f' )
    prtFlux ('riverflow     ', RUN_flx_river      , 'f' )
    prtFlux ('coastal_cpl   ', RUN_flx_coastal_cpl, 'f' )
    prtFlux ('riverf_cpl    ', RUN_flx_river_cpl  , 'f' )
    prtFlux ('river+coastal ', RUN_flx_rivcoa     , 'f' )
    prtFlux ('drainage      ', RUN_flx_drainage   , 'f' )
    prtFlux ('riversret     ', RUN_flx_riversret  , 'f' )
    prtFlux ('runoff        ', RUN_flx_runoff     , 'f' )
    prtFlux ('river in      ', RUN_flx_input      , 'f' )
    prtFlux ('river out     ', RUN_flx_output     , 'f' )
    prtFlux ('river bil     ', RUN_flx_bil        , 'f' )

ATM_flx_budget = -ATM_flx_wbilo + ATM_flx_calving + ATM_flx_runlic #+# ATM_flx_fqfonte #+ RUN_flx_river


echo ('')
#echo ('  Global        {:12.3e} kg | {:12.4f} Sv | {:12.4f} m '.format ( ATM_flx_budget , ATM_flx_budget / dtime_sec*1E-9, ATM_flx_budget /ATM_aire_sea_tot/ATM_rho ))

#echo ('  E-P-R         {:12.3e} kg | {:12.4e} Sv | {:12.4f} m '.format ( ATM_flx_emp      , ATM_flx_emp      / dtime_sec*1E-6/ATM_rho, ATM_flx_emp      /ATM_aire_sea_tot/ATM_rho ))

ATM_flx_toSRF = -ATM_flx_wbilo_ter

echo (' ')
echo ( '\n====================================================================================' )
echo ( f'--  Atmosphere  -- {Title} ' )
echo ( f'Mass begin = {DYN_mas_wat_beg:12.6e} kg | Mass end = {DYN_mas_wat_end:12.6e} kg' )
prtFlux ( 'dmass (atm)  = ', dDYN_mas_wat , 'e', True )
prtFlux ( 'Sum wbilo_*  = ', ATM_flx_wbilo, 'e', True )
prtFlux ( 'E-P          = ', ATM_flx_emp  , 'e', True )
echo ( ' ' )
prtFlux ( 'Water loss atm from wbil_*', ATM_flx_wbilo - dDYN_mas_wat, 'f', True )
echo ( 'Water loss atm = {:12.3e} (rel)  '.format ( (ATM_flx_wbilo - dDYN_mas_wat)/dDYN_mas_wat  ) )

echo (' ')
prtFlux ( 'Water loss atm from E-P', ATM_flx_emp  - dDYN_mas_wat , 'f', True )
echo ( 'Water loss atm = {:12.3e} (rel)  '.format ( (ATM_flx_emp-dDYN_mas_wat)/dDYN_mas_wat  ) )
echo (' ')

ATM_error =  ATM_flx_emp  - dDYN_mas_wat


echo (' ')
echo ( '\n====================================================================================' )

LIC_flx_budget1 = Sprec ( [-ATM_flx_wemp_lic  , -LIC_flx_calving , -LIC_flx_fqfonte] )
LIC_flx_budget2 = Sprec ( [-ATM_flx_wbilo_lic , -LIC_flx_calving , -LIC_flx_fqfonte] )
LIC_flx_budget3 = Sprec ( [-ATM_flx_wbilo_lic , -LIC_flx_runlic] )
LIC_flx_budget4 = Sprec ( [-ATM_flx_wemp_lic  , -LIC_flx_runlic] )

echo ( f'--  LIC  -- {Title} ' )
echo ( f'Mass total   begin = {LIC_mas_wat_beg    :12.6e} kg | Mass end = {LIC_mas_wat_end    :12.6e} kg' )
echo ( f'Mass snow    begin = {LIC_mas_sno_beg    :12.6e} kg | Mass end = {LIC_mas_sno_end    :12.6e} kg' )
echo ( f'Mass qs      begin = {LIC_mas_qs_beg     :12.6e} kg | Mass end = {LIC_mas_qs_end     :12.6e} kg' )
echo ( f'Mass runlic0 begin = {LIC_mas_runlic0_beg:12.6e} kg | Mass end = {LIC_mas_runlic0_end:12.6e} kg' )
prtFlux ( 'dmass (LIC sno)       ', dLIC_mas_sno          , 'f', True, width=45 )
prtFlux ( 'dmass (LIC qs)        ', dLIC_mas_qs           , 'e', True, width=45 )
prtFlux ( 'dmass (LIC wat)       ', dLIC_mas_wat          , 'f', True, width=45 )
prtFlux ( 'dmass (LIC runlic0)   ', dLIC_mas_runlic0      , 'e', True, width=45 )
prtFlux ( 'dmass (LIC total)     ', dLIC_mas_wat          , 'e', True, width=45 )
prtFlux ( 'LIC ATM_flx_wemp_lic  ', ATM_flx_wemp_lic      , 'f', True, width=45 )
prtFlux ( 'LIC LIC_flx_fqfonte   ', LIC_flx_fqfonte       , 'f', True, width=45 )
prtFlux ( 'LIC LIC_flx_calving   ', LIC_flx_calving       , 'f', True, width=45 )
prtFlux ( 'LIC LIC_flx_runofflic ', LIC_flx_runlic        , 'f', True, width=45 )
prtFlux ( 'LIC fqfonte + calving ', LIC_flx_calving+LIC_flx_fqfonte , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 1 ( wemp_lic   - fqcalving - fqfonte)) ', LIC_flx_budget1              , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 2 (-wbilo_lic - fqcalving - fqfonte)   ', LIC_flx_budget2              , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 3 (-wbilo_lic - runofflic*frac_lic)    ', LIC_flx_budget3              , 'f', True, width=45 )
prtFlux ( 'LIC fluxes 3 ( wemp_lic  - runofflic*frac_lic)    ', LIC_flx_budget4              , 'f', True, width=45 )
prtFlux ( 'LIC error 1                                       ', LIC_flx_budget1-dLIC_mas_wat , 'e', True, width=45 )
prtFlux ( 'LIC error 2                                       ', LIC_flx_budget2-dLIC_mas_wat , 'e', True, width=45 )
prtFlux ( 'LIC error 3                                       ', LIC_flx_budget3-dLIC_mas_wat , 'e', True, width=45 )
echo ( 'LIC error (wevap - precip*frac_lic  - fqcalving - fqfonte)    = {:12.4e} (rel) '.format ( (LIC_flx_budget1-dLIC_mas_wat)/dLIC_mas_wat) )
echo ( 'LIC error (-wbilo_lic - fqcalving - fqfonte)                  = {:12.4e} (rel) '.format ( (LIC_flx_budget2-dLIC_mas_wat)/dLIC_mas_wat) )
echo ( 'LIC error (-wbilo_lic - runofflic*frac_lic)                   = {:12.4e} (rel) '.format ( (LIC_flx_budget3-dLIC_mas_wat)/dLIC_mas_wat) )

if SRF :
    echo ( '\n====================================================================================' )
    echo ( f'-- SECHIBA fluxes  -- {Title} ' )

    SRF_flx_rain     = SRF_flux_int ( SRF_rain     )
    SRF_flx_evap     = SRF_flux_int ( SRF_evap     )
    SRF_flx_snowf    = SRF_flux_int ( SRF_snowf    )
    SRF_flx_subli    = SRF_flux_int ( SRF_subli    )
    SRF_flx_transpir = SRF_flux_int ( SRF_transpir )
    SRF_flx_emp      = SRF_flux_int ( SRF_emp      )

    RUN_flx_torouting   = SRF_flux_int  ( RUN_runoff       + RUN_drainage)
    RUN_flx_fromrouting = ONE_flux_int  ( RUN_coastalflow + RUN_riverflow )

    SRF_flx_all =  SRF_flux_int  ( SRF_rain + SRF_snowf - SRF_evap - RUN_runoff - RUN_drainage )

    prtFlux ('rain         ', SRF_flx_rain       , 'f' )
    prtFlux ('evap         ', SRF_flx_evap       , 'f' )
    prtFlux ('snowf        ', SRF_flx_snowf      , 'f' )
    prtFlux ('E-P          ', SRF_flx_emp        , 'f' )
    prtFlux ('subli        ', SRF_flx_subli      , 'f' )
    prtFlux ('transpir     ', SRF_flx_transpir   , 'f' )
    prtFlux ('to routing   ', RUN_flx_torouting  , 'f' )
    prtFlux ('budget       ', SRF_flx_all        , 'f', small=True )

    echo ( '\n------------------------------------------------------------------------------------' )
    echo ( 'Water content in surface ' )
    echo ( f'SRF_mas_wat_beg  = {SRF_mas_wat_beg:12.6e} kg | SRF_mas_wat_end  = {SRF_mas_wat_end:12.6e} kg ' )
    prtFlux ( 'dMass (water srf)', dSRF_mas_wat, 'e', small=True)
    prtFlux ( 'Error            ',  SRF_flx_all-dSRF_mas_wat, 'e', small=True )
    echo ( 'dMass (water srf) = {:12.4e} (rel)   '.format ( (SRF_flx_all-dSRF_mas_wat)/dSRF_mas_wat) )

    echo ( '\n====================================================================================' )
    echo ( f'-- Check ATM vs. SRF -- {Title} ' )
    prtFlux ('E-P ATM       ', ATM_flx_wemp_ter   , 'f' )
    prtFlux ('wbilo ter     ', ATM_flx_wbilo_ter  , 'f' )
    prtFlux ('E-P SRF       ', SRF_flx_emp        , 'f' )
    prtFlux ('SRF/ATM error ', ATM_flx_wbilo_ter - SRF_flx_emp, 'e', True)
    echo ( 'SRF/ATM error {:12.3e} (rel)  '.format ( (ATM_flx_wbilo_ter - SRF_flx_emp)/SRF_flx_emp ) )

    echo ('')
    echo ( '\n====================================================================================' )
    echo ( f'-- RUNOFF fluxes  -- {Title} ' )
    RUN_flx_all = RUN_flx_torouting - RUN_flx_river - RUN_flx_coastal
    prtFlux ('runoff    ', RUN_flx_runoff      , 'f' )
    prtFlux ('drainage  ', RUN_flx_drainage    , 'f' )
    prtFlux ('run+drain ', RUN_flx_torouting   , 'f' )
    prtFlux ('river     ', RUN_flx_river       , 'f' )
    prtFlux ('coastal   ', RUN_flx_coastal     , 'f' )
    prtFlux ('riv+coa   ', RUN_flx_fromrouting , 'f' )
    prtFlux ('budget    ', RUN_flx_all         , 'f' , small=True)

    echo ( '\n------------------------------------------------------------------------------------' )
    echo ( f'Water content in routing+lake  -- {Title} ' )
    echo ( f'RUN_mas_wat_beg  = {RUN_mas_wat_beg:12.6e} kg | RUN_mas_wat_end = {RUN_mas_wat_end:12.6e} kg ' )
    prtFlux ( 'dMass (routing)  ', dRUN_mas_wat+dSRF_mas_lake, 'f', small=True)
    prtFlux ( 'Routing error    ', RUN_flx_all+dSRF_mas_lake-dRUN_mas_wat, 'e', small=True )
    echo ( 'Routing error : {:12.3e} (rel)'.format ( (RUN_flx_all-dSRF_mas_lake-dRUN_mas_wat)/(dRUN_mas_wat+dSRF_mas_lake) ) )

    echo ( '\n------------------------------------------------------------------------------------' )
    echo ( f'Water content in routing  -- {Title} ' )
    echo ( f'RUN_mas_wat_beg  = {RUN_mas_wat_beg:12.6e} kg | RUN_mas_wat_end = {RUN_mas_wat_end:12.6e} kg ' )
    prtFlux ( 'dMass (routing) ', dRUN_mas_wat, 'f', small=True  )
    prtFlux ( 'Routing error   ', RUN_flx_all-dRUN_mas_wat, 'e', small=True)
    echo ( 'Routing error : {:12.3e} (rel)'.format ( (RUN_flx_all-dRUN_mas_wat)/dRUN_mas_wat ) )

echo ( ' ' )
echo ( f'{Title = }' )

echo ( 'SVN Information' )
for kk in SVN.keys():
    print ( SVN[kk] )

## Write the full configuration
params_out = open (FullIniFile, 'w', encoding = 'utf-8')
params = wu.dict2config ( dpar )
params.write ( params_out )
params_out.close ()
