#!/usr/bin/env python3
###
### Script to check water conservation in NEMO
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
SVN = {
    'Author'  : "$Author$",
    'Date'    : "$Date$",
    'Revision': "$Revision$",
    'Id'      : "$Id: $",
    'HeadURL' : "$HeadUrl: $"
    }

###
## Import system modules
import sys
import os
import subprocess
import configparser
from pathlib import Path

## Import needed scientific modules
import numpy  as np
import xarray as xr

## Import local module
import WaterUtils as wu
import nemo

## Read command line arguments
## ---------------------------
print ( "Name of Python script:", sys.argv[0] )
IniFile = sys.argv[1]

# Test existence of IniFile
if not os.path.exists (IniFile ) :
    raise FileExistsError ( f"File not found : {IniFile = }" )

if 'full' in IniFile or 'ATM' in IniFile :
    FullIniFile = IniFile
else :
    FullIniFile = 'ATM_' + IniFile


print ("Output file : ", FullIniFile )

## Experiment parameters
## ---------------------
dpar = wu.ReadConfig ( IniFile )

## Configure all needed parameter from existant parameters
## -------------------------------------------------------
dpar = wu.SetDatesAndFiles ( dpar)

## Output file with water budget diagnostics
## -----------------------------------------
f_out = dpar['Files']['f_out']

## Put dpar values in local namspace
## ---------------------------------
for Section in dpar.keys () : 
    print ( f'\nReading [{Section}]' )
    for VarName in dpar[Section].keys() :
        locals()[VarName] = dpar[Section][VarName]
        print ( f'    {VarName:21} set to : {locals()[VarName]}' )

## Debuging and timer
Timer = wu.functools.partial (wu.Timer, debug=Debug, timer=Timing)

## Useful functions
## ----------------

# Degrades precision
if repr(readPrec) == "<class 'numpy.float64'>" or readPrec == float :
    def rprec (ptab) :
        '''This version does nothing

        rprec may be use to reduce floating precision when reading history files
        '''
        return ptab
else                 :
    def rprec (ptab) :
        '''Return float with a different precision'''
        return ptab.astype(readPrec).astype(float)

def kg2Sv  (val, rho=ATM_RHO) :
    '''From kg to Sverdrup'''
    return val/dtime_sec*1.0e-6/rho

def kg2myear (val, rho=ATM_RHO) :
    '''From kg to m/year'''
    return val/OCE_aire_tot/rho/NbYear

def var2prt (var, small=False, rho=ATM_RHO) :
    '''Formats value for printing'''
    if small :  return var , kg2Sv(var, rho=rho)*1000., kg2myear(var, rho=rho)*1000.
    else     :  return var , kg2Sv(var, rho=rho)      , kg2myear(var, rho=rho)

def prtFlux (Desc, var, Form='F', small=False, rho=ATM_RHO, width=15) :
    '''Pretty print of formattd value'''
    if small :
        if Form in ['f', 'F'] : ff=" {:14.6e} kg | {:12.4f} mSv | {:12.4f} mm/year "
        if Form in ['e', 'E'] : ff=" {:14.6e} kg | {:12.4e} mSv | {:12.4e} mm/year "
    else :
        if Form in ['f', 'F'] : ff=" {:14.6e} kg | {:12.4f} Sv  | {:12.4f} m/year  "
        if Form in ['e', 'E'] : ff=" {:14.6e} kg | {:12.4e} Sv  | {:12.4e} m/year  "
    echo ( (' {:>{width}} = ' +ff).format (Desc, *var2prt(var, small, rho=rho), width=width ) )

def echo (string, end='\n') :
    '''Function to print to stdout *and* output file'''
    print ( str(string), end=end  )
    sys.stdout.flush ()
    f_out.write ( str(string) + end )
    f_out.flush ()

##
## Open history files
## ------------------

d_OCE_his = xr.open_dataset ( file_OCE_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_OCE_sca = xr.open_dataset ( file_OCE_sca, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_ICE_his = xr.open_dataset ( file_ICE_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()

udims = udims={'y':'y', 'x':'x'}
d_OCE_his = nemo.unify_dims ( d_OCE_his, **udims, verbose=False )
d_OCE_sca = nemo.unify_dims ( d_OCE_sca, **udims, verbose=False )
d_ICE_his = nemo.unify_dims ( d_ICE_his, **udims, verbose=False )

echo ( f'{file_OCE_his = }' )
echo ( f'{file_OCE_sca = }' )
echo ( f'{file_ICE_his = }' )

## Compute run length
## ------------------
dtime = ( d_OCE_his.time_counter_bounds.max() - d_OCE_his.time_counter_bounds.min() )
echo ( f'\nRun length : {(dtime/np.timedelta64(1, "D")).values:8.2f} days' )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

## Compute length of each period
## -----------------------------
dtime_per = ( d_OCE_his.time_counter_bounds[:,-1] - d_OCE_his.time_counter_bounds[:,0] )
echo (  f'\nPeriods lengths (days) : {(dtime_per/np.timedelta64(1, "D")).values}' )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_OCE_his.time_counter,] )
dtime_per_sec.attrs['unit'] = 's'

## Number of years
NbYear = dtime_sec / YEAR_LENGTH

## Define restart periods and file names
## -------------------------------------
dpar = wu.SetRestartNames ( dpar, f_out) 

## Put dpar values in local namespace
## ----------------------------------
for Section in dpar.keys () : 
    print ( f'\nReading [{Section}]' )
    for VarName in dpar[Section].keys() :
        locals()[VarName] = dpar[Section][VarName]
        print ( f'    {VarName:21} set to : {locals()[VarName]}' )

echo ( f'{tar_restart_beg = }' )
echo ( f'{tar_restart_end = }' )

echo ( f'{file_OCE_beg}' )
echo ( f'{file_OCE_end}' )
echo ( f'{file_ICE_beg}' )
echo ( f'{file_ICE_end}' )

liste_beg = [file_OCE_beg, file_ICE_beg ]
liste_end = [file_ATM_end, file_ICE_end ]

## Write the full configuration
params_out = open (FullIniFile, 'w', encoding = 'utf-8')
params = wu.dict2config ( dpar )
params.write ( params_out )
params_out.close ()

echo ('\nExtract and rebuild OCE and ICE restarts')
def get_ndomain (zfile) :
    echo ( '--- get domain -- ' )
    echo ( f'{zfile = }' )
    #d_zfile = xr.open_dataset (zfile, decode_times=False).squeeze()
    #ndomain_opa = d_zfile.attrs['DOMAIN_number_total']
    #d_zfile.close ()
    ndomain_opa = subprocess.getoutput ( f'ls {zfile}_*.nc | wc -l' ) #.format()
    echo ( f'{ndomain_opa = }' )
    return int (ndomain_opa)

@Timer
def extract_and_rebuild ( file_name=file_OCE_beg, tar_restart=tar_restart_end, file_dir_comp=FileDirOCE ) :
    '''Extract restart file from tar. Rebuild ocean files if needed'''
    echo ( '---------- extract_and_rebuild --')
    echo ( f'{file_name = }' )
    echo ( f'{tar_restart }' )
    echo ( f'{file_dir_comp = }' )
    error_count = 0
    if os.path.exists ( file_name ) :
        echo ( f'-- File ready : {file_name = }' )
    else :
        echo ( f'-- Extracting {file_name = }' )
        base_resFile = Path (file_name).stem # basename, and remove suffix
        # Try to extract the rebuilded file
        if os.path.exists ( tar_restart ) :
            command =  f'cd {file_dir_comp} ; tar xf {tar_restart} {base_resFile}.nc'
            echo ( f'{command = }' )
            ierr = os.system ( command )
            if ierr != 0 :
                if not os.path.exists ( os.path.join (FileDir, f'{base_resFile}_0000.nc') ):
                    command =  f'cd {file_dir_comp} ; tar xf {tar_restart} {base_resFile}_*.nc'
                    echo ( f'{command = }' )
                    ierr = os.system ( command )
                    if ierr == 0 :
                        echo ( f'tar done : {base_resFile}.nc')
                    else         :
                        if ContinueOnError :
                            error_count += 1
                            echo ( f'****** Command failed : {command}' )
                            echo ( '****** Trying to continue' )
                            echo ( ' ')
                        else :
                            raise OSError ( f'command failed : {command}' )
                    echo ( 'extract ndomain' )
                ndomain_opa = get_ndomain ( os.path.join (file_dir_comp, f'{base_resFile}') )
                command = f'cd {file_dir_comp} ; {rebuild} {base_resFile} {ndomain_opa} ; mv {base_resFile}.nc {FileDir}'
                echo ( f'{command = }' )
                ierr = os.system ( command )
                if ierr == 0 :
                    echo ( f'Rebuild done : {base_resFile}.nc')
                else         :
                    if ContinueOnError :
                        error_count += 1
                        echo ( f'****** Command failed : {command}' )
                        echo ( '****** Trying to continue' )
                        echo ( ' ')
                    else :
                        raise OSError ( f'command failed : {command}' )
            else :
                echo ( f'tar done : {base_resFile}')
                command = f'cd {file_dir_comp} ; mv {base_resFile}.nc {FileDir}'
                echo ( f'{command = }' )
                ierr = os.system ( command )
                if ierr == 0 :
                    echo ( f'command done : {command}' )
                else         :
                    if ContinueOnError :
                        error_count = 1
                        echo ( f'****** Command failed : {command}' )
                        echo ( '****** Trying to continue' )
                        echo ( ' ')
                    else :
                        raise OSError ( f'command failed : {command = }' )
        else :
            echo ( f'****** Tar restart file {tar_restart = } not found ' )
            if ContinueOnError :
                error_count += 1
                echo ( '****** Command failed' )
                echo ( '****** Trying to continue' )
                echo ( ' ')
            else :
                raise OSError ( f'****** tar file not found {tar_restart = } - Stopping' )
    return error_count

ErrorCount = 0

ErrorCount += extract_and_rebuild ( file_name=file_OCE_beg, tar_restart=tar_restart_beg, file_dir_comp=FileDirOCE )
ErrorCount += extract_and_rebuild ( file_name=file_OCE_end, tar_restart=tar_restart_end, file_dir_comp=FileDirOCE )
ErrorCount += extract_and_rebuild ( file_name=file_ICE_beg, tar_restart=tar_restart_beg, file_dir_comp=FileDirICE )
ErrorCount += extract_and_rebuild ( file_name=file_ICE_end, tar_restart=tar_restart_end, file_dir_comp=FileDirICE )

##-- Exit in case of error in the opening file phase
if ErrorCount > 0 :
    echo ( '  ' )
    raise RuntimeError ( f'**** Some files missing - Stopping - {ErrorCount = }' )

echo ('Opening OCE and ICE restart files')
if NEMO == 3.6 :
    d_OCE_beg = xr.open_dataset ( os.path.join (FileDir, file_OCE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_OCE_end = xr.open_dataset ( os.path.join (FileDir, file_OCE_end), decode_times=False, decode_cf=True).squeeze()
    d_ICE_beg = xr.open_dataset ( os.path.join (FileDir, file_ICE_beg), decode_times=False, decode_cf=True).squeeze()
    d_ICE_end = xr.open_dataset ( os.path.join (FileDir, file_ICE_end), decode_times=False, decode_cf=True).squeeze()
if NEMO in [ 4.0, 4.2 ] :
    d_OCE_beg = xr.open_dataset ( os.path.join (FileDir, file_OCE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_OCE_end = xr.open_dataset ( os.path.join (FileDir, file_OCE_end), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_ICE_beg = xr.open_dataset ( os.path.join (FileDir, file_ICE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_ICE_end = xr.open_dataset ( os.path.join (FileDir, file_ICE_end), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()

# Get mask and surfaces
sos = d_OCE_his ['sos'][0].squeeze()
OCE_msk = nemo.lbc_mask ( xr.where ( sos>0., 1., 0. ), cd_type = 'T', nperio=nperio, sval = 0. )

so = sos = d_OCE_his ['sos'][0].squeeze()
OCE_msk3 = nemo.lbc_mask ( xr.where ( so>0., 1., 0. ), cd_type = 'T', nperio=nperio, sval = 0. )

# lbc_mask removes the duplicate points (periodicity and north fold)
OCE_aire = nemo.lbc_mask ( d_OCE_his ['area'] * OCE_msk, cd_type = 'T', nperio=nperio, sval = 0.0 )
ICE_aire = OCE_aire

@Timer
def OCE_stock_int (stock) :
    '''Integrate stock on ocean grid'''
    return wu.Psum (  (stock * OCE_aire ).to_masked_array().ravel() )

@Timer
def ONE_stock_int (stock) :
    '''Sum stock'''
    return wu.Psum ( stock.to_masked_array().ravel() )

@Timer
def OCE_flux_int (flux) :
    '''Integrate flux on oce grid'''
    return wu.Psum  ( (flux * OCE_aire * dtime_per_sec).to_masked_array().ravel() )

@Timer
def ONE_flux_int (flux) :
    '''Integrate flux on oce grid'''
    return wu.Psum ( (flux * dtime_per_sec).to_masked_array().ravel() )

OCE_aire_tot = ONE_stock_int ( OCE_aire )
ICE_aire_tot = ONE_stock_int ( ICE_aire )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- NEMO change in stores (for the records)' )
#-- Note that the total number of days of the run should be diagnosed rather than assumed
#-- Here the result is in Sv
#
#-- Change in ocean volume in freshwater equivalent

OCE_ssh_beg = d_OCE_beg['sshn']
OCE_ssh_end = d_OCE_end['sshn']

OCE_sum_ssh_beg = OCE_stock_int ( OCE_ssh_beg )
OCE_sum_ssh_end = OCE_stock_int ( OCE_ssh_end )

if NEMO == 3.6 :
    OCE_e3tn_beg = d_OCE_beg['fse3t_n']
    OCE_e3tn_end = d_OCE_end['fse3t_n']
    OCE_sum_e3tn_beg = OCE_stock_int ( OCE_e3tn_beg * OCE_msk3)
    OCE_sum_e3tn_end = OCE_stock_int ( OCE_e3tn_end * OCE_msk3)

echo ( f'OCE_sum_ssh_beg = {OCE_sum_ssh_beg:12.6e} m^3  - OCE_sum_ssh_end = {OCE_sum_ssh_end:12.6e} m^3' )
dOCE_ssh_vol = OCE_sum_ssh_end - OCE_sum_ssh_beg
dOCE_ssh_mas = dOCE_ssh_vol * OCE_RHO_LIQ

if NEMO == 3.6 :
    echo ( f'OCE_sum_e3tn_beg = {OCE_sum_e3tn_beg:12.6e} m^3  - OCE_sum_e3tn_end = {OCE_sum_e3tn_end:12.6e} m^3' )
    dOCE_e3tn_vol = OCE_sum_e3tn_end - OCE_sum_e3tn_beg
    dOCE_e3tn_mas = dOCE_e3tn_vol * OCE_RHO_LIQ

dOCE_vol_wat = dOCE_ssh_vol ; dOCE_mas_wat = dOCE_ssh_mas

echo ( f'dOCE vol    = {dOCE_vol_wat             :12.3e} m^3' )
echo ( f'dOCE ssh    = {dOCE_vol_wat/OCE_aire_tot:12.3e} m  ' )
prtFlux ( 'dOCE mass ', dOCE_mas_wat, 'e' )

if NEMO == 3.6 :
    echo ( f'dOCE e3tn   vol    = {dOCE_e3tn_vol:12.3e} m^3' )
    prtFlux ( 'dOCE e3tn   mass', dOCE_e3tn_mas, 'e' )

## Glace et neige
if NEMO == 3.6 :
    ICE_ice_beg = d_ICE_beg['v_i_htc1']+d_ICE_beg['v_i_htc2']+d_ICE_beg['v_i_htc3']+d_ICE_beg['v_i_htc4']+d_ICE_beg['v_i_htc5']
    ICE_ice_end = d_ICE_end['v_i_htc1']+d_ICE_end['v_i_htc2']+d_ICE_end['v_i_htc3']+d_ICE_end['v_i_htc4']+d_ICE_end['v_i_htc5']

    ICE_sno_beg = d_ICE_beg['v_s_htc1']+d_ICE_beg['v_s_htc2']+d_ICE_beg['v_s_htc3']+d_ICE_beg['v_s_htc4']+d_ICE_beg['v_s_htc5']
    ICE_sno_end = d_ICE_end['v_s_htc1']+d_ICE_end['v_s_htc2']+d_ICE_end['v_s_htc3']+d_ICE_end['v_s_htc4']+d_ICE_end['v_s_htc5']

    ICE_pnd_beg = 0.0 ; ICE_pnd_end = 0.0 ; ICE_fzl_beg = 0.0 ; ICE_fzl_end = 0.0

    ICE_mas_wat_beg = ICE_ice_beg*ICE_RHO_ICE + ICE_sno_beg*ICE_RHO_SNO
    ICE_mas_wat_end = ICE_ice_end*ICE_RHO_ICE + ICE_sno_end*ICE_RHO_SNO

if NEMO in [ 4.0, 4.2 ] :
    ICE_ice_beg = d_ICE_beg['v_i']  ; ICE_ice_end = d_ICE_end['v_i']
    ICE_sno_beg = d_ICE_beg['v_s']  ; ICE_sno_end = d_ICE_end['v_s']
    ICE_pnd_beg = d_ICE_beg['v_ip'] ; ICE_pnd_end = d_ICE_end['v_ip'] # Ice ponds
    ICE_fzl_beg = d_ICE_beg['v_il'] ; ICE_fzl_end = d_ICE_end['v_il'] # Frozen Ice Ponds

    ICE_mas_wat_beg =  OCE_stock_int ( d_ICE_beg['snwice_mass'] )
    ICE_mas_wat_end =  OCE_stock_int ( d_ICE_end['snwice_mass'] )

ICE_vol_ice_beg = OCE_stock_int ( ICE_ice_beg )
ICE_vol_ice_end = OCE_stock_int ( ICE_ice_end )

ICE_vol_sno_beg = OCE_stock_int ( ICE_sno_beg )
ICE_vol_sno_end = OCE_stock_int ( ICE_sno_end )

ICE_vol_pnd_beg = OCE_stock_int ( ICE_pnd_beg )
ICE_vol_pnd_end = OCE_stock_int ( ICE_pnd_end )

ICE_vol_fzl_beg = OCE_stock_int ( ICE_fzl_beg )
ICE_vol_fzl_end = OCE_stock_int ( ICE_fzl_end )

#-- Converting to freswater volume
dICE_vol_ice = ICE_vol_ice_end - ICE_vol_ice_beg
dICE_mas_ice = dICE_vol_ice * ICE_RHO_ICE

dICE_vol_sno = ICE_vol_sno_end - ICE_vol_sno_beg
dICE_mas_sno = dICE_vol_sno * ICE_RHO_SNO

dICE_vol_pnd = ICE_vol_pnd_end - ICE_vol_pnd_beg
dICE_mas_pnd = dICE_vol_pnd * ICE_RHO_PND

dICE_vol_fzl= ICE_vol_fzl_end - ICE_vol_fzl_beg
dICE_mas_fzl = dICE_vol_fzl * ICE_RHO_PND

if NEMO == 3.6 :
    dICE_mas_wat = dICE_mas_ice + dICE_mas_sno
    dSEA_mas_wat = dOCE_mas_wat + dICE_mas_ice + dICE_mas_sno

if NEMO in [4.0, 4.2 ] :
    dICE_mas_wat = ICE_mas_wat_end - ICE_mas_wat_beg
    dSEA_mas_wat = dOCE_mas_wat + dICE_mas_wat

echo ( f'ICE_vol_ice_beg = {ICE_vol_ice_beg:12.6e} m^3 | ICE_vol_ice_end = {ICE_vol_ice_end:12.6e} m^3' )
echo ( f'ICE_vol_sno_beg = {ICE_vol_sno_beg:12.6e} m^3 | ICE_vol_sno_end = {ICE_vol_sno_end:12.6e} m^3' )
echo ( f'ICE_vol_pnd_beg = {ICE_vol_pnd_beg:12.6e} m^3 | ICE_vol_pnd_end = {ICE_vol_pnd_end:12.6e} m^3' )
echo ( f'ICE_vol_fzl_beg = {ICE_vol_fzl_beg:12.6e} m^3 | ICE_vol_fzl_end = {ICE_vol_fzl_end:12.6e} m^3' )

echo ( f'dICE_vol_ice   = {dICE_vol_ice:12.3e} m^3' )
echo ( f'dICE_vol_sno   = {dICE_vol_sno:12.3e} m^3' )
echo ( f'dICE_vol_pnd   = {dICE_vol_pnd:12.3e} m^3' )
echo ( f'dICE_mas_ice   = {dICE_mas_ice:12.3e} m^3' )
echo ( f'dICE_mas_sno   = {dICE_mas_sno:12.3e} m^3' )
echo ( f'dICE_mas_pnd   = {dICE_mas_pnd:12.3e} m^3' )
echo ( f'dICE_mas_fzl   = {dICE_mas_fzl:12.3e} m^3' )

echo ( '\n------------------------------------------------------------')
echo ( 'Change in water content (ocean + ice) ' )
prtFlux ( 'dMass (ocean)', dSEA_mas_wat, 'e', True )


###
### And now, working on fluxes !!

# if coupled:
# emp_oce = evap - precip (including blowing snow) - calving
# emp_ice = evap - precip (excluding blowing snow)
# emp_ice = wfx_spr(<0) + wfx_sub(>0) + wfx_snw_sub(v4.0.6) - wfx_err_sub(<0)
# runoffs = rivers + icebergs
# empmr = emp_oce - wfx_ice - wfx_snw - wfx_pnd(v4.0.6) - wfx_err_sub - runoffs
# doce+ice = - evap + precip + calving (emp_oce)
#            + rivers + icebergs       (friver+iceberg ou runoffs)
#            + iceshelf                (iceshelf)
#            - emp_ice                 (emp_ice)
# dice = - emp_ice - wfx_snw - wfx_ice - wfx_pnd(v4.0.6) - wfx_err_sub

#OCE_emp = evap - precip (including blowing snow) - calving
#ICE_emp = wfx_spr(<0) + wfx_sub(>0) + wfx_snw_sub(v4.0.6) - wfx_err_sub(<0)

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- Checks in NEMO - from budget_modipsl.sh (Clément Rousset)' )

# Read variable and computes integral over space and time
OCE_empmr      = rprec (d_OCE_his['wfo']     )    ; OCE_mas_empmr    = OCE_flux_int ( OCE_empmr    )
OCE_wfob       = rprec (d_OCE_his['wfob']    )    ; OCE_mas_wfob     = OCE_flux_int ( OCE_wfob     )
OCE_emp_oce    = rprec (d_OCE_his['emp_oce'] )    ; OCE_mas_emp_oce  = OCE_flux_int ( OCE_emp_oce  )
OCE_emp_ice    = rprec (d_OCE_his['emp_ice'] )    ; OCE_mas_emp_ice  = OCE_flux_int ( OCE_emp_ice  )
OCE_iceshelf   = rprec (d_OCE_his['iceshelf'])    ; OCE_mas_iceshelf = OCE_flux_int ( OCE_iceshelf )
OCE_calving    = rprec (d_OCE_his['calving'] )    ; OCE_mas_calving  = OCE_flux_int ( OCE_calving  )
OCE_iceberg    = rprec (d_OCE_his['iceberg'] )    ; OCE_mas_iceberg  = OCE_flux_int ( OCE_iceberg  )
OCE_friver     = rprec (d_OCE_his['friver']  )    ; OCE_mas_friver   = OCE_flux_int ( OCE_friver   )
OCE_runoffs    = rprec (d_OCE_his['runoffs'] )    ; OCE_mas_runoffs  = OCE_flux_int ( OCE_runoffs  )
if NEMO in [ 4.0, 4.2 ] :
    OCE_wfxice     = rprec (d_OCE_his['vfxice']) ; OCE_mas_wfxice   = OCE_flux_int ( OCE_wfxice )
    OCE_wfxsnw     = rprec (d_OCE_his['vfxsnw']) ; OCE_mas_wfxsnw   = OCE_flux_int ( OCE_wfxsnw )
    OCE_wfxsub     = rprec (d_OCE_his['vfxsub']) ; OCE_mas_wfxsub   = OCE_flux_int ( OCE_wfxsub )
if NEMO == 3.6 :
    OCE_wfxice     = rprec (d_OCE_his['vfxice'])/nemo.RDAY*ICE_RHO_ICE ; OCE_mas_wfxice   = OCE_flux_int ( OCE_wfxice )
    OCE_wfxsnw     = rprec (d_OCE_his['vfxsnw'])/nemo.RDAY*ICE_RHO_SNO ; OCE_mas_wfxsnw   = OCE_flux_int ( OCE_wfxsnw )
    OCE_wfxsub     = rprec (d_OCE_his['vfxsub'])/nemo.RDAY*ICE_RHO_SNO ; OCE_mas_wfxsub   = OCE_flux_int ( OCE_wfxsub )
# Additional checks
OCE_evap_oce   = rprec (d_OCE_his['evap_ao_cea']) ; OCE_mas_evap_oce   = OCE_flux_int ( OCE_evap_oce )
ICE_evap_ice   = rprec (d_OCE_his['subl_ai_cea']) ; ICE_mas_evap_ice   = OCE_flux_int ( ICE_evap_ice )
OCE_snow_oce   = rprec (d_OCE_his['snow_ao_cea']) ; OCE_mas_snow_oce   = OCE_flux_int ( OCE_snow_oce )
OCE_snow_ice   = rprec (d_OCE_his['snow_ai_cea']) ; OCE_mas_snow_ice   = OCE_flux_int ( OCE_snow_ice )
OCE_rain       = rprec (d_OCE_his['rain']       ) ; OCE_mas_rain       = OCE_flux_int ( OCE_rain     )
ICE_wfxsub_err = rprec (d_ICE_his['vfxsub_err'] ) ; ICE_mas_wfxsub_err = OCE_flux_int ( ICE_wfxsub_err )
if NEMO in [ 4.0, 4.2 ] :
    ICE_wfxpnd     = rprec (d_ICE_his['vfxpnd']    ) ; ICE_mas_wfxpnd     = OCE_flux_int ( ICE_wfxpnd     )
    ICE_wfxsnw_sub = rprec (d_ICE_his['vfxsnw_sub']) ; ICE_mas_wfxsnw_sub = OCE_flux_int ( ICE_wfxsnw_sub )
    ICE_wfxsnw_pre = rprec (d_ICE_his['vfxsnw_pre']) ; ICE_mas_wfxsnw_pre = OCE_flux_int ( ICE_wfxsnw_pre )
if NEMO == 3.6 :
    ICE_wfxpnd = 0.0 ; ICE_mas_wfxpnd = 0.0
    ICE_wfxsnw_sub = rprec (d_ICE_his['vfxsub'])/nemo.RDAY*ICE_RHO_SNO  ; ICE_mas_wfxsnw_sub = OCE_flux_int ( ICE_wfxsnw_sub )
    ICE_wfxsnw_pre = rprec (d_ICE_his['vfxspr'])/nemo.RDAY*ICE_RHO_SNO  ; ICE_mas_wfxsnw_pre = OCE_flux_int ( ICE_wfxsnw_pre )

OCE_wfcorr    = rprec (d_OCE_his['wfcorr']) ; OCE_mas_wfcorr  = OCE_flux_int ( OCE_wfcorr )
if OCE_relax  :
    # ssr and fwb are included in emp=>empmr but not in emp_oce (outputed by sea-ice)
    OCE_vflx_fwb = rprec (d_OCE_his['vflx_fwb']) ; OCE_mas_vflx_fwb   = OCE_flux_int ( OCE_vflx_fwb )
    OCE_vflx_ssr = rprec (d_OCE_his['vflx_ssr']) ; OCE_mas_vflx_ssr   = OCE_flux_int ( OCE_vflx_ssr )
else :
    OCE_fwb = 0.0 ; OCE_mas_fwb = 0.0
    OCE_ssr = 0.0 ; OCE_mas_ssr = 0.0
if OCE_icb :
    OCE_berg_icb    = rprec (d_OCE_his['berg_floating_melt']) ; OCE_mas_berg_icb = OCE_flux_int ( OCE_berg_icb    )
    OCE_calving_icb = rprec (d_OCE_his['calving_icb']       ) ; OCE_mas_calv_icb = OCE_flux_int ( OCE_calving_icb )
else :
    OCE_berg_icb = 0. ; OCE_mas_berg_icb = 0.
    OCE_calv_icb = 0. ; OCE_mas_calv_icb = 0.

OCE_mas_emp = OCE_mas_emp_oce - OCE_mas_wfxice - OCE_mas_wfxsnw - ICE_mas_wfxpnd - ICE_mas_wfxsub_err
OCE_mas_all = OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceshelf

echo ('\n-- Fields:' )
prtFlux ('OCE+ICE budget ', OCE_mas_all       , 'e', True)
prtFlux ('  EMPMR        ', OCE_mas_empmr     , 'e', True)
prtFlux ('  WFOB         ', OCE_mas_wfob      , 'e', True)
prtFlux ('  EMP_OCE      ', OCE_mas_emp_oce   , 'e', True)
prtFlux ('  EMP_ICE      ', OCE_mas_emp_ice   , 'e', True)
prtFlux ('  EMP          ', OCE_mas_emp       , 'e', True)
prtFlux ('  ICEBERG      ', OCE_mas_iceberg   , 'e', True)
prtFlux ('  ICESHELF     ', OCE_mas_iceshelf  , 'e', True)
prtFlux ('  CALVING      ', OCE_mas_calving   , 'e', True)
prtFlux ('  FRIVER       ', OCE_mas_friver    , 'e', True)
prtFlux ('  RUNOFFS      ', OCE_mas_runoffs   , 'e', True)
prtFlux ('  WFXICE       ', OCE_mas_wfxice    , 'e', True)
prtFlux ('  WFXSNW       ', OCE_mas_wfxsnw    , 'e', True)
prtFlux ('  WFXSUB       ', OCE_mas_wfxsub    , 'e', True)
prtFlux ('  WFXPND       ', ICE_mas_wfxpnd    , 'e', True)
prtFlux ('  WFXSNW_SUB   ', ICE_mas_wfxsnw_sub, 'e', True)
prtFlux ('  WFXSNW_PRE   ', ICE_mas_wfxsnw_pre, 'e', True)
prtFlux ('  WFXSUB_ERR   ', ICE_mas_wfxsub_err, 'e', True)
prtFlux ('  EVAP_OCE     ', OCE_mas_evap_oce  , 'e'      )
prtFlux ('  EVAP_ICE     ', ICE_mas_evap_ice  , 'e', True)
prtFlux ('  SNOW_OCE     ', OCE_mas_snow_oce  , 'e', True)
prtFlux ('  SNOW_ICE     ', OCE_mas_snow_ice  , 'e', True)
prtFlux ('  RAIN         ', OCE_mas_rain      , 'e'      )
prtFlux ('  FWB          ', OCE_mas_fwb       , 'e', True)
prtFlux ('  SSR          ', OCE_mas_ssr       , 'e', True)
prtFlux ('  WFCORR       ', OCE_mas_wfcorr    , 'e', True)
prtFlux ('  BERG_ICB     ', OCE_mas_berg_icb  , 'e', True)
prtFlux ('  CALV_ICB     ', OCE_mas_calv_icb  , 'e', True)

echo (' ')
if Coupled :
    prtFlux ( 'Bilan ocean :', -OCE_mas_emp_oce - OCE_mas_emp_ice + OCE_mas_runoffs + OCE_mas_iceshelf, 'e', True )
else :
    prtFlux ( 'Bilan ocean :', -OCE_mas_emp_oce - OCE_mas_emp_ice + OCE_mas_runoffs + OCE_mas_iceberg + OCE_mas_calving + OCE_mas_iceshelf, 'e', True )


echo     ('\n===>  Comparing ===>' )
echo     ('  WFX OCE                     = -empmr + iceshelf                                            = {:12.5e} (kg) '.format ( -OCE_mas_empmr + OCE_mas_iceshelf) )
echo     ('     versus d OCE                                                                            = {:12.5e} (kg) '.format ( dOCE_mas_wat) )
echo     ('  WFX ICE+SNW+PND 1           = emp_ice - wfxice - wfxsnw - wfxpnd - wfxsub_err              = {:12.5e} (kg) '.format ( -OCE_mas_emp_ice - OCE_mas_wfxice - OCE_mas_wfxsnw - ICE_mas_wfxpnd - ICE_mas_wfxsub_err) )
echo     ('  WFX ICE+SNW+PND 2           = -emp_ice + empmr - emp_oce + runoffs                         = {:12.5e} (kg) '.format ( -OCE_mas_emp_ice + OCE_mas_empmr - OCE_mas_emp_oce + OCE_mas_runoffs ))
echo     ('     versus d ICE+SNW+PND                                                                    = {:12.5e} (kg) '.format ( dICE_mas_wat))  # Manque PND ?
if Coupled :
    echo ('  WFX OCE+ICE+SNW+PND         = -emp_oce - emp_ice + runoffs + iceshelf                      = {:12.5e} (kg) '.format ( -OCE_mas_emp_oce - OCE_mas_emp_ice + OCE_mas_runoffs + OCE_mas_iceshelf))
else :
    echo ('  WFX OCE+ICE+SNW+PND         = -emp_oce - emp_ice + runoffs + iceberg + calving + iceshelf  = {:12.5e} (kg) '.format ( -OCE_mas_emp_oce - OCE_mas_emp_ice + OCE_mas_runoffs + OCE_mas_iceberg + OCE_mas_calving + OCE_mas_iceshelf))
echo     ('     versus d OCE+ICE+SNW+PND                                                                = {:12.5e} (kg) '.format ( dSEA_mas_wat  )) # Manque PND

echo ('\n===> Leaks ===>')
echo ('   Leak OCE             = dOCE_mas_wat + empmr - iceshelf                                                 = {:12.3e} (kg) '.format ( dOCE_mas_wat + OCE_mas_empmr - OCE_mas_iceshelf) )
echo ('                        = (dOCE_mas_wat + empmr - iceshelf)/abs(dOCE_mas_wat)                             = {:12.1e} (-)  '.format ( (dOCE_mas_wat + OCE_mas_empmr - OCE_mas_iceshelf )  / abs (dOCE_mas_wat)  ) )
echo ('   Leak ICE+SNW+PND 1   = dICE_mas_wat + emp_ice + wfxice + wfxsnw + wfxpnd + wfxsub_err                  = {:12.3e} (kg) '.format ( dICE_mas_wat + OCE_mas_emp_ice + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err ) )
echo ('                        = (dICE_mas_wat + emp_ice + wfxice + wfxsnw + wfxpnd + wfxsub_err )/dICE_mas_wat  = {:12.1e} (-)  '.format ( (dICE_mas_wat + OCE_mas_emp_ice + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err)/abs(dICE_mas_wat)) )
echo ('   Leak ICE+SNW+PND 2   = dICE_mas_wat + emp_ice - empmr + emp_oce - runoffs                              = {:12.3e} (kg) '.format ( dICE_mas_wat + OCE_mas_emp_ice - OCE_mas_empmr + OCE_mas_emp_oce - OCE_mas_runoffs ))
echo ('                        = (dICE_mas_wat - empmr  + emp_oce - runoffs)/abs(dICE_mas_wat)                   = {:12.1e} (-)  '.format ( (dICE_mas_wat - OCE_mas_empmr  + OCE_mas_emp_oce - OCE_mas_runoffs)/abs(dICE_mas_wat)) )
echo ('   Leak OCE+ICE+SNW+PND =  d(ICE+OCE)_mas_wat + emp_oce + emp_ice - runoffs - iceshelf                    = {:12.3e} (kg) '.format ( dSEA_mas_wat + OCE_mas_emp_oce +OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceshelf ))
echo ('                        = (dSEA_mas_wat + emp_oce + emp_ice - runoffs - iceshelf)*/abs(dSEA_mas_wat)      = {:12.1e} (-)  '.format ( (dSEA_mas_wat + OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceshelf)/abs(dSEA_mas_wat) ) )

prtFlux ('   Leak OCE             ', ( dOCE_mas_wat + OCE_mas_empmr - OCE_mas_iceshelf) , 'e', True )
prtFlux ('   Leak ICE+SNW+PND     ', ( dICE_mas_wat + OCE_mas_emp_ice + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err ) , 'e', True )
prtFlux ('   Leak OCE+ICE+SNW+PND ',  ( dSEA_mas_wat + OCE_mas_emp_oce +OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceshelf ) , 'e', True )

# check if emp     = emp_ice + emp_oce - calving
#          emp_ice = wfxsub + wfxsnw_sub + wfxsnw_pre - wfxsub_err

echo     ('\n===> Checks ===>' )
echo     ('   Check EMPMR           = empmr - emp_oce + runoffs + wfxice + wfxsnw + wfxpnd + wfxsub_err = {:12.5e} (kg) '.format ( OCE_mas_empmr - OCE_mas_emp_oce + OCE_mas_runoffs + OCE_mas_wfxice + OCE_mas_wfxsnw + ICE_mas_wfxpnd + ICE_mas_wfxsub_err ))
if Coupled :
    echo ('   Check EMP_OCE         = emp_oce + fwb + ssr - evap_oce + rain + snow_oce + calving        = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_fwb + OCE_mas_ssr - OCE_mas_evap_oce + OCE_mas_rain + OCE_mas_snow_oce + OCE_mas_calving ))
else :
    echo ('   Check EMP_OCE         = emp_oce + ssr + fwb - evap_oce + rain + snow_oce                  = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_ssr + OCE_mas_fwb - OCE_mas_evap_oce + OCE_mas_rain + OCE_mas_snow_oce ))
echo     ('   Check EMP_ICE         = emp_ice - evap_ice + snow_ice                                     = {:12.5e} (kg) '.format ( OCE_mas_emp_ice - ICE_mas_evap_ice + OCE_mas_snow_ice ))
echo     ('   Check EMP_ICE vs WFX  = emp_ice - wfxsub - wfxsnw_sub - wfxsnw_pre + wfxsub_err           = {:12.5e} (kg) '.format ( OCE_mas_emp_ice - OCE_mas_wfxsub - ICE_mas_wfxsnw_sub - ICE_mas_wfxsnw_pre + ICE_mas_wfxsub_err ))

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- Computations in the PDF note of Clément Rousset')
echo ( 'Freshwater budget of the ice-ocean system          = emp_oce + emp_ice - runoffs - iceberg - iceshelf                = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_runoffs - OCE_mas_iceberg - OCE_mas_iceshelf ))
echo ( 'Freshwater budget of the ice-ocean system          = emp_oce + emp_ice - friver  - iceberg - iceshelf                = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_emp_ice - OCE_mas_friver  - OCE_mas_iceberg - OCE_mas_iceshelf ))
echo ( 'Freshwater budget in the ocean = ocean mass change = emp_oce - vfxice - ( vfxsnw + vfxsub_err ) - runoffs - iceshelf = {:12.5e} (kg) '.format ( OCE_mas_emp_oce - OCE_mas_wfxice - ( OCE_mas_wfxsnw + ICE_mas_wfxsub_err ) - OCE_mas_runoffs - OCE_mas_iceshelf ))
echo ( 'Freshwater budget in the ocean = ocean mass change = emp_oce - vfxice - ( vfxsnw + vfxsub_err ) - friver  - iceshelf = {:12.5e} (kg) '.format ( OCE_mas_emp_oce - OCE_mas_wfxice - ( OCE_mas_wfxsnw + ICE_mas_wfxsub_err ) - OCE_mas_friver  - OCE_mas_iceshelf ))
echo ( 'Freshwater budget in the ice = ice mass change     = vfxice + ( vfxsnw + vfxsub + vfxspr )                           = {:12.5e} (kg) '.format ( OCE_mas_wfxice  + OCE_mas_wfxsnw + OCE_mas_wfxsub + ICE_mas_wfxsnw_pre ))
echo ( 'Freshwater flux at the interface [ice/snow]-ocean  = vfxice + ( vfxsnw + vfxsub_err )                                = {:12.5e} (kg) '.format ( OCE_mas_wfxsub  + ICE_mas_wfxsnw_pre ))
echo ( 'Freshwater flux at the interface [ice/snow]-atm    = ( vfxsub + vfxspr )                                             = {:12.5e} (kg) '.format ( OCE_mas_emp_ice + ICE_mas_wfxsub_err ))
echo ( 'Freshwater flux at the interface [ice/snow]-atm    = emp_ice + vfxsub_err                                            = {:12.5e} (kg) '.format ( OCE_mas_emp_ice + ICE_mas_wfxsub_err ))
echo ( 'Freshwater flux at the interface ocean-atm         = emp_oce + calving - vfxsub_err                                  = {:12.5e} (kg) '.format ( OCE_mas_emp_oce + OCE_mas_calving - ICE_mas_wfxsub_err ))

echo ( "scsshtot   : global_average_sea_level_change                            = {:12.3e} (m) ".format ( np.sum (d_OCE_sca['scsshtot']  ).values  ) )
echo ( "scsshtot   : global_average_sea_level_change                            = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['scsshtot']  ).values  * OCE_aire_tot*OCE_RHO_LIQ  ) )
echo ( "bgvolssh   : drift in global mean ssh volume wrt timestep 1             = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['bgvolssh']  ).values  * 1e9 * OCE_RHO_LIQ  ) )
echo ( "bgvole3t   : drift in global mean volume variation (e3t) wrt timestep 1 = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['bgvole3t']  ).values  * 1e9 * OCE_RHO_LIQ  ) )
echo ( "bgfrcvol   : global mean volume from forcing                            = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['bgfrcvol']  ).values  * 1e9 * OCE_RHO_LIQ  ) )
echo ( "ibgvol_tot : global mean ice volume                                     = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['ibgvol_tot']).values  * 1e9 * OCE_RHO_LIQ  ) )
echo ( "sbgvol_tot : global mean snow volume                                    = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['sbgvol_tot']).values  * 1e9 * OCE_RHO_LIQ  ) )
echo ( "ibgvolume  : drift in ice/snow volume (equivalent ocean volume)         = {:12.3e} (kg)".format ( np.sum (d_OCE_sca['ibgvolume'] ).values  * 1e9 * OCE_RHO_LIQ  ) )

echo ( ' ' )
echo ( f'{Title = }' )

echo ( 'SVN Information' )
for kk in SVN.keys():
    print ( SVN[kk] )

## Write the full configuration
params_out = open (FullIniFile, 'w', encoding = 'utf-8')
params = wu.dict2config ( dpar )
params.write ( params_out )
params_out.close ()
